unit DemoMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  IniClass, StdCtrls, ExtCtrls, Spin, Mask, Buttons, Menus;

type

{ TDemoIniClass }

  TDemoEnum = (deFirst, deSecond, deThird);
  TDemoEnums = set of TDemoEnum;

  TClassProp = class(TPersistent)
  private
    FClassIntProp: Integer;
  published
    property ClassIntProp: Integer read FClassIntProp write FClassIntProp
      default 5;
  end;

  TDemoIniClass = class(TIniClass)
  private
    FIntProp: Integer;
    FCharProp: Char;
    FEnumProp: TDemoEnum;
    FFloatProp: Extended;
    FStringProp: string;
    FSetProp: TDemoEnums;
    FClassProp: TClassProp;
    FStringsProp: TStrings;
  published
    constructor Create;
    destructor Destroy; override;
    property IntProp: Integer read FIntProp write FIntProp;
    property CharProp: Char read FCharProp write FCharProp;
    property EnumProp: TDemoEnum read FEnumProp write FEnumProp;
    property FloatProp: Extended read FFloatProp write FFloatProp;
    property StringProp: string read FStringProp write FStringProp;
    property SetProp: TDemoEnums read FSetProp write FSetProp;
    property ClassProp: TClassProp read FClassProp write FClassProp;
    property StringsProp: TStrings read FStringsProp write FStringsProp;
  end;

  TDemoMainForm = class(TForm)
    GroupBox1: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    SaveTypeGroup: TRadioGroup;
    GroupBox2: TGroupBox;
    IntPropEdit: TSpinEdit;
    Label3: TLabel;
    Panel1: TPanel;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Label4: TLabel;
    CharPropBox: TComboBox;
    EnumBox: TRadioGroup;
    Label5: TLabel;
    FloatEdit: TMaskEdit;
    Label6: TLabel;
    StringEdit: TEdit;
    GroupBox3: TGroupBox;
    FirstBox: TCheckBox;
    SecondBox: TCheckBox;
    ThirdBox: TCheckBox;
    GroupBox5: TGroupBox;
    StringsEdit: TEdit;
    StringsBox: TListBox;
    AddButton: TSpeedButton;
    DeleteButton: TSpeedButton;
    Label7: TLabel;
    ClassIntEdit: TSpinEdit;
    MainMenu1: TMainMenu;
    Help1: TMenuItem;
    About1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure StringsEditChange(Sender: TObject);
    procedure AddButtonClick(Sender: TObject);
    procedure StringsBoxClick(Sender: TObject);
    procedure DeleteButtonClick(Sender: TObject);
    procedure About1Click(Sender: TObject);
  private
    FIniClass: TDemoIniClass;
    procedure Load;
    procedure Save;
    procedure EnableButtons;
  end;

var
  DemoMainForm: TDemoMainForm;

implementation

uses AboutFrm;

{$R *.DFM}

{ TDemoIniClass }

constructor TDemoIniClass.Create;
begin
  inherited Create;
  FIntProp := 5;
  FCharProp := 'A';
  FEnumProp := deFirst;
  FFloatProp := 5;
  FStringProp := 'IniClass demo';
  FSetProp := [deFirst, deSecond];
  FClassProp := TClassProp.Create;
  FStringsProp := TStringList.Create;
  FStringsProp.Add('First string');
  FStringsProp.Add('Second string')
end;

destructor TDemoIniClass.Destroy;
begin
  FStringsProp.Free;
  FClassProp.Free;
  inherited Destroy
end;

const
  SaveTypes: array[0..1] of TSaveType = (stRegistry, stIniFile);

procedure TDemoMainForm.Load;
const
  EnumIndex: array[TDemoEnum] of Integer = (0, 1, 2);

  function GetCharIndex(Strings: TStrings; Ch: Char): Integer;
  var
    I: Integer;
  begin
    Result := -1;
    for I := 0 to Strings.Count - 1 do
      if Strings[I] = Ch then
      begin
        Result := I;
        Break
      end
  end;

begin
  with FIniClass do
  begin
    SaveType := SaveTypes[SaveTypeGroup.ItemIndex];
    Load;
    IntPropEdit.Value := IntProp;
    CharPropBox.ItemIndex := GetCharIndex(CharPropBox.Items, CharProp);
    EnumBox.ItemIndex := EnumIndex[EnumProp];
    FloatEdit.Text := FloatToStr(FloatProp);
    StringEdit.Text := StringProp;
    FirstBox.Checked := deFirst in SetProp;
    SecondBox.Checked := deSecond in SetProp;
    ThirdBox.Checked := deThird in SetProp;
    ClassIntEdit.Value := ClassProp.ClassIntProp;
    StringsBox.Items.Assign(StringsProp);
    EnableButtons
  end
end;

procedure TDemoMainForm.Save;
const
  IndexEnum: array[0..2] of TDemoEnum = (deFirst, deSecond, deThird);
begin
  with FIniClass do
  begin
    SaveType := SaveTypes[SaveTypeGroup.ItemIndex];
    IntProp := IntPropEdit.Value;
    CharProp := CharPropBox.Text[1];
    EnumProp := IndexEnum[EnumBox.ItemIndex];
    FloatProp := StrToFloat(FloatEdit.EditText);
    StringProp := StringEdit.Text;
    if FirstBox.Checked then SetProp := SetProp + [deFirst]
    else
      SetProp := SetProp - [deFirst];
    if SecondBox.Checked then SetProp := SetProp + [deSecond]
    else
      SetProp := SetProp - [deSecond];
    if ThirdBox.Checked then SetProp := SetProp + [deThird]
    else
      SetProp := SetProp - [deThird];
    ClassProp.ClassIntProp := ClassIntEdit.Value;      
    StringsProp.Assign(StringsBox.Items);
    Save
  end
end;

procedure TDemoMainForm.EnableButtons;
begin
  AddButton.Enabled := (StringsEdit.Text <> '') and
    (StringsBox.Items.IndexOf(StringsEdit.Text) = -1);
  DeleteButton.Enabled := StringsBox.ItemIndex <> -1
end;

procedure TDemoMainForm.FormCreate(Sender: TObject);
begin
  FIniClass := TDemoIniClass.Create;
  FloatEdit.EditMask := '!99999' + DecimalSeparator + '9;1;0';
  Load
end;

procedure TDemoMainForm.FormDestroy(Sender: TObject);
begin
  Save
end;

procedure TDemoMainForm.Button1Click(Sender: TObject);
begin
  Load
end;

procedure TDemoMainForm.Button2Click(Sender: TObject);
begin
  Save
end;

procedure TDemoMainForm.Button3Click(Sender: TObject);
begin
  Close
end;

procedure TDemoMainForm.StringsEditChange(Sender: TObject);
begin
  EnableButtons
end;

procedure TDemoMainForm.AddButtonClick(Sender: TObject);
begin
  if StringsEdit.Text <> '' then
    with StringsBox do
    begin
      Items.Add(StringsEdit.Text);
      ItemIndex := Items.Count - 1
    end;
  StringsBoxClick(nil)
end;

procedure TDemoMainForm.StringsBoxClick(Sender: TObject);
begin
  with StringsBox do
    if ItemIndex <> -1 then
      StringsEdit.Text := Items[ItemIndex]
    else
      StringsEdit.Text := '';
  EnableButtons
end;

procedure TDemoMainForm.DeleteButtonClick(Sender: TObject);
begin
  with StringsBox do
    if ItemIndex <> -1 then
    begin
      Items.Delete(ItemIndex);
      ItemIndex := ItemIndex - 1
    end;
  StringsBoxClick(nil)
end;

procedure TDemoMainForm.About1Click(Sender: TObject);
begin
  with TAbout.Create(Application) do
    try
      ShowModal
    finally
      Free
    end
end;

end.
