{*******************************************************}
{                                                       }
{              Memory Inspector project                 }
{                                                       }
{              Windows, not using VCL                   }
{                                                       }
{           Copyright (c) 2000 Serge Gusev              }
{                                                       }
{*******************************************************}


unit FreeWin;

interface

{$H+}

uses Messages, Windows, SysUtils, Classes, Graphics, Controls, Forms, StdCtrls;

type

  { TFreeWinControl }

  TFreeWinControl = class(TObject)
  private
    FDefWndProc: Pointer;
    FObjectInstance: Pointer;
    FHandle: HWND;
    FParent: TFreeWinControl;
    FParentWindow: HWND;
    FChildren: TList;
    FCaption: string;
    FLeft: Integer;
    FTop: Integer;
    FWidth: Integer;
    FHeight: Integer;
    FFontData: TFontData;
    FVisible: Boolean;
    procedure HandleNeeded;
    function GetHandle: HWND;
    procedure SetParent(const Value: TFreeWinControl);
    function GetChild(Index: Integer): TFreeWinControl;
    function GetChildrenCount: Integer;
    procedure SetCaption(const Value: string);
    procedure SetHeight(const Value: Integer);
    procedure SetLeft(const Value: Integer);
    procedure SetTop(const Value: Integer);
    procedure SetWidth(const Value: Integer);
    procedure SetFontData(const Value: TFontData);
    procedure SetVisible(const Value: Boolean);
    procedure InsertChild(Child: TFreeWinControl);
    procedure RemoveChild(Child: TFreeWinControl);
    function PrecedingWindow(Control: TFreeWinControl): HWnd;
    procedure CreateFontHandle;
    procedure WMCommand(var Message: TWMCommand); message WM_COMMAND;
  protected
    procedure WndProc(var Message: TMessage); virtual;
    procedure CreateHandle; virtual;
    procedure CreateParams(var Params: TCreateParams); virtual;
    procedure CreateWindowHandle(const Params: TCreateParams); virtual;
    procedure CreateSubClass(var Params: TCreateParams;
      ControlClassName: PChar);
    procedure CreateWnd; virtual;
    procedure RecreateWnd;
    procedure DestroyHandle;
  public
    constructor Create(AParent: TFreeWinControl; AParentWindow: HWND); virtual;
    destructor Destroy; override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
    procedure Invalidate;
    property Handle: HWND read GetHandle;
    property Parent: TFreeWinControl read FParent write SetParent;
    property Child[Index: Integer]: TFreeWinControl read GetChild;
    property ChildrenCount: Integer read GetChildrenCount;
    property Caption: string read FCaption write SetCaption;
    property Left: Integer read FLeft write SetLeft;
    property Top: Integer read FTop write SetTop;
    property Width: Integer read FWidth write SetWidth;
    property Height: Integer read FHeight write SetHeight;
    property FontData: TFontData read FFontData write SetFontData;
    property Visible: Boolean read FVisible write SetVisible;
  end;

  { TFreeForm }

  TFreeForm = class(TFreeWinControl)
  private
    FModalResult: TModalResult;
    procedure WMPaint(var Message: TMessage); message WM_PAINT;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure WndProc(var Message: TMessage); override;
  public
    constructor Create(AParent: TFreeWinControl; AParentWindow: HWND); override;
    procedure Show(IsScreenCenter: Boolean = True);
    function ShowModal(IsScreenCenter: Boolean = True): TModalResult;
    procedure Hide;
    procedure Close;
  end;

  { TFreeStatic }

  TFreeStatic = class(TFreeWinControl)
  protected
    procedure CreateParams(var Params: TCreateParams); override;
  public
    constructor Create(AParent: TFreeWinControl; AParentWindow: HWND); override;
  end;

  { TFreeButton }

  TFreeButton = class(TFreeWinControl)
  private
    FOnClick: TNotifyEvent;
    procedure CNCommand(var Message: TWMCommand); message CN_COMMAND;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure Click; virtual;
  public
    constructor Create(AParent: TFreeWinControl; AParentWindow: HWND); override;
    property OnClick: TNotifyEvent read FOnClick write FOnClick;
  end;

  { TFreeGroupBox }

  TFreeGroupBox = class(TFreeWinControl)
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure WndProc(var Message: TMessage); override;
  public
    constructor Create(AParent: TFreeWinControl; AParentWindow: HWND); override;
  end;

  { TFreeComboBox }

  TFreeComboBoxStyle = (csDropDown, csSimple, csDropDownList, csOwnerDrawFixed);

  TFreeComboBox = class(TFreeWinControl)
  private
    FStyle: TFreeComboBoxStyle;
    FItems: TStringList;
    FDropDownCount: Cardinal;
    FDroppedDown: Boolean;
    FItemIndex: Integer;
    FOnClick: TNotifyEvent;
    procedure SetStyle(const Value: TFreeComboBoxStyle);
    procedure SetItems(const Value: TStringList);
    procedure SetDropDownCount(Value: Cardinal);
    function GetDroppedDown: Boolean;
    procedure SetDroppedDown(Value: Boolean);
    function GetItemIndex: Integer;
    procedure SetItemIndex(const Value: Integer);
    procedure CNCommand(var Message: TWMCommand); message CN_COMMAND;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure Click; virtual;
  public
    constructor Create(AParent: TFreeWinControl; AParentWindow: HWND); override;
    destructor Destroy; override;
    property Style: TFreeComboBoxStyle read FStyle write SetStyle;
    property Items: TStringList read FItems write SetItems;
    property DropDownCount: Cardinal read FDropDownCount write SetDropDownCount;
    property DroppedDown: Boolean read GetDroppedDown write SetDroppedDown;
    property ItemIndex: Integer read GetItemIndex write SetItemIndex;
    property OnClick: TNotifyEvent read FOnClick write FOnClick;
  end;

  { TFreeListBox }

  TFreeListBox = class(TFreeWinControl)
  private
    FStyle: TListBoxStyle;
    FItems: TStringList;
    FItemIndex: Integer;
    FOnClick: TNotifyEvent;
    procedure SetStyle(const Value: TListBoxStyle);
    procedure SetItems(const Value: TStringList);
    function GetItemIndex: Integer;
    procedure SetItemIndex(const Value: Integer);
    procedure CNCommand(var Message: TWMCommand); message CN_COMMAND;
  protected
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CreateHandle; override;
    procedure Click; virtual;
  public
    constructor Create(AParent: TFreeWinControl; AParentWindow: HWND); override;
    destructor Destroy; override;
    property Style : TListBoxStyle read FStyle write SetStyle;
    property Items: TStringList read FItems write SetItems;
    property ItemIndex: Integer read GetItemIndex write SetItemIndex;
    property OnClick: TNotifyEvent read FOnClick write FOnClick;
  end;

implementation

uses
  Consts;

{ TFreeWinControl }

constructor TFreeWinControl.Create(AParent: TFreeWinControl; AParentWindow: HWND);
begin
  inherited Create;
  FObjectInstance := MakeObjectInstance(WndProc);
  FDefWndProc :=  @DefWindowProc;
  FHandle := 0;
  FFontData := DefFontData;
  FFontData.Height := -13;
  Parent := AParent;
  if FParent = nil then
    FParentWindow := AParentWindow
  else
    FParentWindow := FParent.Handle;
  FChildren := nil;
  FCaption := '';
  FVisible := True
end;

destructor TFreeWinControl.Destroy;
begin
  if FFontData.Handle <> 0 then
    DeleteObject(FFontData.Handle);
  while FChildren <> nil do Child[0].Free;
  Parent := nil;
  if FHandle <> 0 then
    DestroyWindow(FHandle);
  FreeObjectInstance(FObjectInstance);
  inherited Destroy
end;

procedure TFreeWinControl.HandleNeeded;
begin
  if FHandle = 0 then
  begin
    if FParent <> nil then FParent.HandleNeeded;
    CreateHandle
  end
end;

function TFreeWinControl.GetHandle: HWND;
begin
  HandleNeeded;
  Result := FHandle
end;

function TFreeWinControl.GetChild(Index: Integer): TFreeWinControl;
begin
  if FChildren = nil then Result := nil
  else
    Result := TFreeWinControl(FChildren[Index])
end;

function TFreeWinControl.GetChildrenCount: Integer;
begin
  if FChildren = nil then Result := 0
  else
    Result := FChildren.Count
end;

procedure TFreeWinControl.SetCaption(const Value: string);
begin
  if Value <> FCaption then
  begin
    FCaption := Value;
    SendMessage(Handle, WM_SETTEXT, 0, Longint(PChar(FCaption)))
  end
end;

function TFreeWinControl.PrecedingWindow(Control: TFreeWinControl): HWnd;
var
  I: Integer;
begin
  if FChildren <> nil then
    for I := FChildren.IndexOf(Control) + 1 to FChildren.Count - 1 do
    begin
      Result := TFreeWinControl(FChildren[I]).FHandle;
      if Result <> 0 then Exit
    end;
  Result := HWND_TOP
end;

procedure TFreeWinControl.CreateFontHandle;
var
  LogFont: TLogFont;
begin
  if FFontData.Handle <> 0 then
    DeleteObject(FFontData.Handle);
  with LogFont, FFontData do
  begin
    lfHeight := Height;
    lfWidth := 0;
    lfEscapement := 0;
    lfOrientation := 0;
    if fsBold in Style then
      lfWeight := FW_BOLD
    else
      lfWeight := FW_NORMAL;
    lfItalic := Byte(fsItalic in Style);
    lfUnderline := Byte(fsUnderline in Style);
    lfStrikeOut := Byte(fsStrikeOut in Style);
    lfCharSet := Byte(Charset);
    if AnsiCompareText(Name, 'Default') = 0 then
      StrPCopy(lfFaceName, DefFontData.Name)
    else
      StrPCopy(lfFaceName, Name);
    lfQuality := DEFAULT_QUALITY;
    lfOutPrecision := OUT_DEFAULT_PRECIS;
    lfClipPrecision := CLIP_DEFAULT_PRECIS;
    case Pitch of
      fpVariable: lfPitchAndFamily := VARIABLE_PITCH;
      fpFixed: lfPitchAndFamily := FIXED_PITCH
    else
      lfPitchAndFamily := DEFAULT_PITCH
    end;
    Handle := CreateFontIndirect(LogFont)
  end
end;

procedure TFreeWinControl.WMCommand(var Message: TWMCommand);
begin
  Message.Result := SendMessage(Message.Ctl, CN_COMMAND,
    TMessage(Message).wParam, TMessage(Message).lParam) 
end;

procedure TFreeWinControl.WndProc(var Message: TMessage);
begin
  if Message.Msg = WM_WINDOWPOSCHANGED then
    with PWindowPos(Message.lParam)^ do
    begin
      FLeft := x;
      FTop := y;
      FWidth := cx;
      FHeight := cy
    end;
  with Message do
    Result := CallWindowProc(FDefWndProc, FHandle, Msg, wParam, lParam);
  Dispatch(Message)
end;

procedure TFreeWinControl.CreateHandle;
var
  PrecWindow: HWND;
begin
  if FHandle = 0 then
  begin
    CreateWnd;
    if FParent = nil then PrecWindow := HWND_TOP
    else
      PrecWindow := FParent.PrecedingWindow(Self);
    SetWindowPos(FHandle, PrecWindow, 0, 0, 0, 0,
       SWP_NOMOVE + SWP_NOSIZE + SWP_NOACTIVATE)
  end
end;

procedure TFreeWinControl.CreateParams(var Params: TCreateParams);
begin
  FillChar(Params, SizeOf(Params), 0);
  with Params do
  begin
    Caption := PChar(FCaption);
    Style := WS_CHILD or WS_CLIPSIBLINGS or WS_CLIPCHILDREN or WS_TABSTOP or
      WS_VISIBLE;
    X := FLeft;
    Y := FTop;
    Width := FWidth;
    Height := FHeight;
    WndParent := FParentWindow;
    WindowClass.style := CS_VREDRAW + CS_HREDRAW + CS_DBLCLKS;
    WindowClass.lpfnWndProc := @DefWindowProc;
    WindowClass.hCursor := LoadCursor(0, IDC_ARROW);
    WindowClass.hbrBackground := 0;
    WindowClass.hInstance := HInstance;
    StrPCopy(WinClassName, ClassName)
  end
end;

procedure TFreeWinControl.CreateWindowHandle(const Params: TCreateParams);
begin
  with Params do
    FHandle := CreateWindowEx(ExStyle, WinClassName, Caption, Style,
      X, Y, Width, Height, WndParent, 0, WindowClass.hInstance, Param);
  SetWindowLong(FHandle, GWL_WNDPROC, Integer(FObjectInstance))
end;

procedure TFreeWinControl.CreateWnd;
var
  Params: TCreateParams;
  TempClass: TWndClass;
  ClassRegistered: Boolean;
begin
  CreateParams(Params);
  with Params do
  begin
    if (WndParent = 0) and (Style and WS_CHILD <> 0) then
      raise EInvalidOperation.CreateFmt(SParentRequired, [ClassName]);
    FDefWndProc := WindowClass.lpfnWndProc;
    ClassRegistered := GetClassInfo(WindowClass.hInstance, WinClassName, TempClass);
    if not ClassRegistered then
    begin
      WindowClass.lpfnWndProc := FDefWndProc;
      WindowClass.lpszClassName := WinClassName;
      if Windows.RegisterClass(WindowClass) = 0 then RaiseLastWin32Error
    end;
    CreateWindowHandle(Params);
    if FHandle = 0 then RaiseLastWin32Error
  end;
  CreateFontHandle;
  SendMessage(Handle, WM_SETFONT, FFontData.Handle, 1)
end;

procedure TFreeWinControl.RecreateWnd;
var
  WasFocused: Boolean;
begin
  if FHandle <> 0 then
  begin
    WasFocused := GetFocus = FHandle;
    DestroyHandle;
    HandleNeeded;
    if WasFocused and (FHandle <> 0) then SetFocus(FHandle)
  end
end;

procedure TFreeWinControl.DestroyHandle;
var
  I: Integer;
begin
  if FHandle <> 0 then
  begin
    if FChildren <> nil then
      for I := 0 to ChildrenCount - 1 do
        Child[I].DestroyHandle;
    DestroyWindow(FHandle);
    FHandle := 0
  end
end;

procedure TFreeWinControl.CreateSubClass(var Params: TCreateParams;
  ControlClassName: PChar);
const
  CS_OFF = CS_OWNDC or CS_CLASSDC or CS_PARENTDC or CS_GLOBALCLASS;
  CS_ON = CS_VREDRAW or CS_HREDRAW;
var
  SaveInstance: THandle;
begin
  if ControlClassName <> nil then
    with Params do
    begin
      SaveInstance := WindowClass.hInstance;
      if not GetClassInfo(HInstance, ControlClassName, WindowClass) and
        not GetClassInfo(0, ControlClassName, WindowClass) and
        not GetClassInfo(MainInstance, ControlClassName, WindowClass) then
        GetClassInfo(WindowClass.hInstance, ControlClassName, WindowClass);
      WindowClass.hInstance := SaveInstance;
      WindowClass.style := WindowClass.style and not CS_OFF or CS_ON
    end
end;

procedure TFreeWinControl.InsertChild(Child: TFreeWinControl);
begin
  if FChildren = nil then
    FChildren := TList.Create;
  if FChildren.IndexOf(Child) = -1 then
    FChildren.Add(Child)
end;

procedure TFreeWinControl.RemoveChild(Child: TFreeWinControl);
begin
  if FChildren <> nil then
  begin
    FChildren.Remove(Child);
    if FChildren.Count = 0 then
    begin
      FChildren.Free;
      FChildren := nil
    end
  end
end;

procedure TFreeWinControl.SetParent(const Value: TFreeWinControl);
begin
  if Value <> FParent then
  begin
    if FParent <> nil then FParent.RemoveChild(Self);
    FParent := Value;
    if FParent <> nil then
    begin
      FParent.InsertChild(Self);
      FParentWindow := FParent.Handle;
      HandleNeeded;
      if FHandle <> 0 then
        Windows.SetParent(FHandle, FParentWindow)
    end
  end
end;

procedure TFreeWinControl.SetLeft(const Value: Integer);
begin
  if Value <> FLeft then
    SetBounds(Value, FTop, FWidth, FHeight)
end;

procedure TFreeWinControl.SetTop(const Value: Integer);
begin
  if Value <> FTop then
    SetBounds(FLeft, Value, FWidth, FHeight)
end;

procedure TFreeWinControl.SetWidth(const Value: Integer);
begin
  if Value <> FWidth then
    SetBounds(FLeft, FTop, Value, FHeight)
end;

procedure TFreeWinControl.SetHeight(const Value: Integer);
begin
  if Value <> FHeight then
    SetBounds(FLeft, FTop, FWidth, Value)
end;

procedure TFreeWinControl.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
var
  PrecWindow: HWND;
begin
  if (ALeft <> FLeft) or (ATop <> FTop) or
    (AWidth <> FWidth) or (AHeight <> FHeight) then
  begin
    FLeft := ALeft;
    FTop := ATop;
    FWidth := AWidth;
    FHeight := AHeight;
    if FParent = nil then PrecWindow := HWND_TOP
    else
      PrecWindow := FParent.PrecedingWindow(Self);
    SetWindowPos(Handle, PrecWindow, FLeft, FTop, FWidth, FHeight, 0)
  end
end;

procedure TFreeWinControl.Invalidate;
var
  R: TRect;
begin
  R := Rect(0, 0, FWidth, FHeight);
  InvalidateRect(Handle, @R, False)
end;

procedure TFreeWinControl.SetFontData(const Value: TFontData);
begin
  if FFontData.Handle <> 0 then
    DeleteObject(FFontData.Handle);
  FFontData := Value;
  CreateFontHandle
end;

procedure TFreeWinControl.SetVisible(const Value: Boolean);
const
  CmdShow: array[Boolean] of Integer = (SW_HIDE, SW_SHOW);
begin
  if Value <> FVisible then
  begin
    FVisible := Value;
    if Value and (FParent <> nil) then
      FParent.Visible := True;
    ShowWindow(Handle, CmdShow[Value])
  end
end;

{ TFreeForm }

constructor TFreeForm.Create(AParent: TFreeWinControl; AParentWindow: HWND);
begin
  inherited Create(AParent, AParentWindow);
  FVisible := False;
  FWidth := 400;
  FHeight := 300;
  FModalResult := -1
end;

procedure TFreeForm.WMPaint(var Message: TMessage);
var
  DC: HDC;
  Brush: HBRUSH;
begin
  DC := GetDC(FHandle);
  try
    Brush := GetSysColorBrush(COLOR_BTNFACE);
    try
      FillRect(DC, Rect(0, 0, FWidth, FHeight), Brush)
    finally
      DeleteObject(Brush)
    end
  finally
    ReleaseDC(FHandle, DC)
  end;
  inherited
end;

procedure TFreeForm.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  with Params do
  begin
    Style := WS_CLIPSIBLINGS or WS_CLIPCHILDREN or WS_POPUP or WS_CAPTION
      or WS_SYSMENU;
    ExStyle := WS_EX_CONTROLPARENT or WS_EX_DLGMODALFRAME or WS_EX_WINDOWEDGE;
    with WindowClass do
      Style := Style or CS_SAVEBITS or CS_BYTEALIGNWINDOW
  end
end;

procedure TFreeForm.WndProc(var Message: TMessage);
begin
  if Message.Msg = WM_CLOSE then
  begin
    if FModalResult = 0 then FModalResult := mrCancel;
    Hide;
    Message.Msg := WM_NULL
  end;
  inherited WndProc(Message)
end;

procedure TFreeForm.Show(IsScreenCenter: Boolean = True);
begin
  if IsScreenCenter then
  begin
    FLeft := (GetSystemMetrics(SM_CXFULLSCREEN) - FWidth) div 2;
    FTop := (GetSystemMetrics(SM_CYFULLSCREEN) - FHeight) div 2
  end;
  FVisible := True;
  Win32Check(SetWindowPos(Handle, HWND_TOP, FLeft, FTop, FWidth, FHeight, SWP_SHOWWINDOW))
end;

function TFreeForm.ShowModal(IsScreenCenter: Boolean = True): TModalResult;
var
  WindowsList: Pointer;
  ActiveWindow: HWND;
  Msg: TMsg;
begin
  Hide;
  FModalResult := 0;
  if GetCapture <> 0 then SendMessage(GetCapture, WM_CANCELMODE, 0, 0);
  ReleaseCapture;
  ActiveWindow := GetActiveWindow;
  WindowsList := DisableTaskWindows(0);
  try
    Show(IsScreenCenter);
    FModalResult := 0;
    repeat
      if GetMessage(Msg, 0, 0, 0) then
      begin
        TranslateMessage(Msg);
        DispatchMessage(Msg)
      end
      else
        FModalResult := mrCancel
    until FModalResult <> 0
  finally
    EnableTaskWindows(WindowsList);
    if ActiveWindow <> 0 then
      SetActiveWindow(ActiveWindow)
  end;
  Result := FModalResult;
  Hide
end;

procedure TFreeForm.Hide;
begin
  Visible := False
end;

procedure TFreeForm.Close;
begin
  if FHandle <> 0 then
    PostMessage(FHandle, WM_CLOSE, 0, 0) 
end;

{ TFreeStatic }

constructor TFreeStatic.Create(AParent: TFreeWinControl; AParentWindow: HWND);
begin
  inherited Create(AParent, AParentWindow);
  FWidth := 75;
  FHeight := 16
end;

procedure TFreeStatic.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  CreateSubClass(Params, 'STATIC')
end;

{ TFreeButton }

constructor TFreeButton.Create(AParent: TFreeWinControl; AParentWindow: HWND);
begin
  inherited Create(AParent, AParentWindow);
  FHeight := 25;
  FWidth := 75
end;

procedure TFreeButton.CNCommand(var Message: TWMCommand);
begin
  if Message.NotifyCode = BN_CLICKED then Click;
  inherited
end;

procedure TFreeButton.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  CreateSubClass(Params, 'BUTTON')
end;

procedure TFreeButton.Click;
begin
  if Assigned(FOnClick) then FOnClick(Self)
end;

{ TFreeGroupBox }

constructor TFreeGroupBox.Create(AParent: TFreeWinControl; AParentWindow: HWND);
begin
  inherited Create(AParent, AParentWindow);
  FWidth := 185;
  FHeight := 145
end;

procedure TFreeGroupBox.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  CreateSubClass(Params, 'BUTTON');
  with Params do
  begin
    Style := Style or WS_CLIPSIBLINGS or WS_CLIPCHILDREN or BS_GROUPBOX;
    ExStyle := ExStyle or WS_EX_CONTROLPARENT
  end
end;

procedure TFreeGroupBox.WndProc(var Message: TMessage);
var
  DC: HDC;
  Brush: HBRUSH;
  R: TRect;
begin
  if Message.Msg = WM_PAINT then
  begin
    R := Rect(0, 0, FWidth, FHeight);
    DC := GetDC(FHandle);
    try
      Brush := GetSysColorBrush(COLOR_BTNFACE);
      try
        FillRect(DC, R, Brush)
      finally
        DeleteObject(Brush)
      end
    finally
      ReleaseDC(FHandle, DC)
    end;
    InvalidateRect(FHandle, @R, False)
  end;
  inherited WndProc(Message)
end;

{ TFreeComboBox }

const
  ItemHeight = 16;

constructor TFreeComboBox.Create(AParent: TFreeWinControl; AParentWindow: HWND);
begin
  FStyle := csDropDown;
  inherited Create(AParent, AParentWindow);
  FWidth := 145;
  FHeight := 24;
  FItems := TStringList.Create;
  FDropDownCount := 8;
  FDroppedDown := False;
  FItemIndex := -1
end;

destructor TFreeComboBox.Destroy;
begin
  FItems.Free;
  inherited Destroy
end;

procedure TFreeComboBox.SetStyle(const Value: TFreeComboBoxStyle);
begin
  if Value <> FStyle then
  begin
    FStyle := Value;
    RecreateWnd
  end
end;

procedure TFreeComboBox.SetItems(const Value: TStringList);
var
  I: Integer;
begin
  if Value = nil then FItems.Clear
  else
    FItems.Assign(Value);
  SendMessage(Handle, CB_RESETCONTENT, 0, 0);
  for I := 0 to FItems.Count - 1 do
    SendMessage(Handle, CB_ADDSTRING, 0, Longint(PChar(FItems[I])));
  FItemIndex := -1  
end;

procedure TFreeComboBox.SetDropDownCount(Value: Cardinal);
begin
  if Value = 0 then Value := 1;
  if Value <> FDropDownCount then
  begin
    FDropDownCount := Value;
    if FDroppedDown then
    begin
      DroppedDown := False;
      DroppedDown := True
    end
  end
end;

function TFreeComboBox.GetDroppedDown: Boolean;
begin
  FDroppedDown := Boolean(SendMessage(Handle, CB_GETDROPPEDSTATE, 0, 0));
  Result := FDroppedDown
end;

procedure TFreeComboBox.SetDroppedDown(Value: Boolean);
begin
  if Value <> FDroppedDown then
  begin
    FDroppedDown := Value;
    if Value then FHeight := 24 + FDropDownCount * ItemHeight + 4
    else
      FHeight := 24;
    Value := Boolean(SendMessage(Handle, CB_GETDROPPEDSTATE, 0, 0));
    if Value <> FDroppedDown then
      SendMessage(Handle,  CB_SHOWDROPDOWN, Integer(Value), 0);
    SetWindowPos(Handle, 0, FLeft, FTop, FWidth, FHeight, 0)
  end
end;

function TFreeComboBox.GetItemIndex: Integer;
begin
  FItemIndex := SendMessage(Handle, CB_GETCURSEL, 0, 0);
  Result := FItemIndex
end;

procedure TFreeComboBox.SetItemIndex(const Value: Integer);
begin
  if Value <> FItemIndex then
  begin
    FItemIndex := Value;
    SendMessage(Handle, CB_SETCURSEL, Value, 0);
    Click
  end
end;

procedure TFreeComboBox.CNCommand(var Message: TWMCommand);
begin
  if Message.NotifyCode = CBN_DROPDOWN then DroppedDown := True;
  if Message.NotifyCode = CBN_CLOSEUP then DroppedDown := False;
  inherited
end;

procedure TFreeComboBox.CreateParams(var Params: TCreateParams);
const
  ComboBoxStyles: array[TFreeComboBoxStyle] of DWORD = (
    CBS_DROPDOWN, CBS_SIMPLE, CBS_DROPDOWNLIST,
    CBS_DROPDOWNLIST or CBS_OWNERDRAWFIXED);
begin
  inherited CreateParams(Params);
  CreateSubClass(Params, 'COMBOBOX');
  with Params do
    Style := Style or WS_VSCROLL or CBS_HASSTRINGS or CBS_AUTOHSCROLL or
      ComboBoxStyles[FStyle]
end;

procedure TFreeComboBox.Click;
begin
  if Assigned(FOnClick) then FOnClick(Self)
end;

{ TFreeListBox }

constructor TFreeListBox.Create(AParent: TFreeWinControl; AParentWindow: HWND);
begin
  inherited Create(AParent, AParentWindow);
  FWidth := 121;
  FHeight := 97;
  FStyle := lbStandard;
  FItems := TStringList.Create;
  FItemIndex := -1
end;

destructor TFreeListBox.Destroy;
begin
  FItems.Free;
  inherited Destroy
end;

procedure TFreeListBox.SetStyle(const Value: TListBoxStyle);
begin
  if Value <> FStyle then
  begin
    FStyle := Value;
    RecreateWnd
  end
end;

procedure TFreeListBox.SetItems(const Value: TStringList);
var
  I: Integer;
begin
  if Value = nil then FItems.Clear
  else
    FItems.Assign(Value);
  SendMessage(Handle, LB_RESETCONTENT, 0, 0);
  for I := 0 to FItems.Count - 1 do
    SendMessage(Handle, LB_ADDSTRING, 0, Longint(PChar(FItems[I])));
  FItemIndex := -1  
end;

function TFreeListBox.GetItemIndex: Integer;
begin
  FItemIndex := SendMessage(Handle, LB_GETCURSEL, 0, 0);
  Result := FItemIndex
end;

procedure TFreeListBox.SetItemIndex(const Value: Integer);
begin
  if Value <> FItemIndex then
  begin
    FItemIndex := Value;
    SendMessage(Handle,  LB_SETCURSEL, Value, 0);
    Click
  end
end;

procedure TFreeListBox.CNCommand(var Message: TWMCommand);
begin
  if Message.NotifyCode = LBN_SELCHANGE then
    ItemIndex := SendMessage(Handle, LB_GETCURSEL, 0, 0);
  inherited
end;

procedure TFreeListBox.CreateParams(var Params: TCreateParams);
const
  Styles: array[TListBoxStyle] of DWORD =
    (0, LBS_OWNERDRAWFIXED, LBS_OWNERDRAWVARIABLE);
begin
  inherited CreateParams(Params);
  CreateSubClass(Params, 'LISTBOX');
  with Params do
  begin
    Style := Style or WS_HSCROLL or WS_VSCROLL or LBS_HASSTRINGS or LBS_NOTIFY or
      Styles[FStyle];
    ExStyle := ExStyle or WS_EX_CLIENTEDGE
  end
end;

procedure TFreeListBox.CreateHandle;
begin
  inherited CreateHandle;
  if FItems <> nil then SetItems(FItems)
end;

procedure TFreeListBox.Click;
begin
  if Assigned(FOnClick) then FOnClick(Self)
end;

end.
