unit GeometryFunctions;

interface

uses
  GeometryTypes;

type
  TGeometryFunctionsMode = (gfOptimal, gfX87Pascal, gfX87, gf3DNow);

procedure SetGeometryFunctionsMode(Mode: TGeometryFunctionsMode);
function GetGeometryFunctionsMode : TGeometryFunctionsMode;
procedure SetOptimalGeometryFunctionsMode;
function GetOptimalGeometryFunctionsMode : TGeometryFunctionsMode;

var
//  Vector Algebra:

// *****************************************************************************
// AddVector:
// *****************************************************************************

  AddVector2f : procedure(const v1,v2: TVector2f; out res: TVector2f); register;
  AddVector2d : procedure(const v1,v2: TVector2d; out res: TVector2d); register;
  AddVector3f : procedure(const v1,v2: TVector3f; out res: TVector3f); register;
  AddVector3d : procedure(const v1,v2: TVector3d; out res: TVector3d); register;
  AddVector4f : procedure(const v1,v2: TVector4f; out res: TVector4f); register;
  AddVector4d : procedure(const v1,v2: TVector4d; out res: TVector4d); register;
  AddHomVector3f : procedure(const v1,v2: TVector3f; out res: TVector3f); register;
  AddHomVector3d : procedure(const v1,v2: TVector3d; out res: TVector3d); register;
  AddHomVector4f : procedure(const v1,v2: TVector4f; out res: TVector4f); register;
  AddHomVector4d : procedure(const v1,v2: TVector4d; out res: TVector4d); register;

// *****************************************************************************
// SubVector:
// *****************************************************************************

  SubVector2f : procedure(const v1,v2: TVector2f; out res: TVector2f); register;
  SubVector2d : procedure(const v1,v2: TVector2d; out res: TVector2d); register;
  SubVector3f : procedure(const v1,v2: TVector3f; out res: TVector3f); register;
  SubVector3d : procedure(const v1,v2: TVector3d; out res: TVector3d); register;
  SubVector4f : procedure(const v1,v2: TVector4f; out res: TVector4f); register;
  SubVector4d : procedure(const v1,v2: TVector4d; out res: TVector4d); register;
  SubHomVector3f : procedure(const v1,v2: TVector3f; out res: TVector3f); register;
  SubHomVector3d : procedure(const v1,v2: TVector3d; out res: TVector3d); register;
  SubHomVector4f : procedure(const v1,v2: TVector4f; out res: TVector4f); register;
  SubHomVector4d : procedure(const v1,v2: TVector4d; out res: TVector4d); register;

// *****************************************************************************
// AddScalarVector:
// *****************************************************************************

  AddScalarVector2f : procedure(const v: TVector2f; s: TScalarf; out res: TVector2f); register;
  AddScalarVector2d : procedure(const v: TVector2d; s: TScalard; out res: TVector2d); register;
  AddScalarVector3f : procedure(const v: TVector3f; s: TScalarf; out res: TVector3f); register;
  AddScalarVector3d : procedure(const v: TVector3d; s: TScalard; out res: TVector3d); register;
  AddScalarVector4f : procedure(const v: TVector4f; s: TScalarf; out res: TVector4f); register;
  AddScalarVector4d : procedure(const v: TVector4d; s: TScalard; out res: TVector4d); register;

// *****************************************************************************
// SubScalarVector:
// *****************************************************************************

  SubScalarVector2f : procedure(const v: TVector2f; s: TScalarf; out res: TVector2f); register;
  SubScalarVector2d : procedure(const v: TVector2d; s: TScalard; out res: TVector2d); register;
  SubScalarVector3f : procedure(const v: TVector3f; s: TScalarf; out res: TVector3f); register;
  SubScalarVector3d : procedure(const v: TVector3d; s: TScalard; out res: TVector3d); register;
  SubScalarVector4f : procedure(const v: TVector4f; s: TScalarf; out res: TVector4f); register;
  SubScalarVector4d : procedure(const v: TVector4d; s: TScalard; out res: TVector4d); register;

// *****************************************************************************
// MulVector:
// *****************************************************************************

  MulVector2f : procedure(const v1,v2: TVector2f; out res: TVector2f); register;
  MulVector2d : procedure(const v1,v2: TVector2d; out res: TVector2d); register;
  MulVector3f : procedure(const v1,v2: TVector3f; out res: TVector3f); register;
  MulVector3d : procedure(const v1,v2: TVector3d; out res: TVector3d); register;
  MulVector4f : procedure(const v1,v2: TVector4f; out res: TVector4f); register;
  MulVector4d : procedure(const v1,v2: TVector4d; out res: TVector4d); register;

// *****************************************************************************
// DivVector:
// *****************************************************************************

  DivVector2f : procedure(const v1,v2: TVector2f; out res: TVector2f); register;
  DivVector2d : procedure(const v1,v2: TVector2d; out res: TVector2d); register;
  DivVector3f : procedure(const v1,v2: TVector3f; out res: TVector3f); register;
  DivVector3d : procedure(const v1,v2: TVector3d; out res: TVector3d); register;
  DivVector4f : procedure(const v1,v2: TVector4f; out res: TVector4f); register;
  DivVector4d : procedure(const v1,v2: TVector4d; out res: TVector4d); register;

// *****************************************************************************
// MulScalarVector:
// *****************************************************************************

  MulScalarVector2f : procedure(const v: TVector2f; s: TScalarf; out res: TVector2f); register;
  MulScalarVector2d : procedure(const v: TVector2d; s: TScalard; out res: TVector2d); register;
  MulScalarVector3f : procedure(const v: TVector3f; s: TScalarf; out res: TVector3f); register;
  MulScalarVector3d : procedure(const v: TVector3d; s: TScalard; out res: TVector3d); register;
  MulScalarVector4f : procedure(const v: TVector4f; s: TScalarf; out res: TVector4f); register;
  MulScalarVector4d : procedure(const v: TVector4d; s: TScalard; out res: TVector4d); register;

// *****************************************************************************
// DivScalarVector:
// *****************************************************************************

  DivScalarVector2f : procedure(const v: TVector2f; s: TScalarf; out res: TVector2f); register;
  DivScalarVector2d : procedure(const v: TVector2d; s: TScalard; out res: TVector2d); register;
  DivScalarVector3f : procedure(const v: TVector3f; s: TScalarf; out res: TVector3f); register;
  DivScalarVector3d : procedure(const v: TVector3d; s: TScalard; out res: TVector3d); register;
  DivScalarVector4f : procedure(const v: TVector4f; s: TScalarf; out res: TVector4f); register;
  DivScalarVector4d : procedure(const v: TVector4d; s: TScalard; out res: TVector4d); register;

// *****************************************************************************
// ScaleVector:
// *****************************************************************************

  ScaleVector2f : procedure(const v: TVector2f; s: TScalarf; out res: TVector2f); register;
  ScaleVector2d : procedure(const v: TVector2d; s: TScalard; out res: TVector2d); register;
  ScaleVector3f : procedure(const v: TVector3f; s: TScalarf; out res: TVector3f); register;
  ScaleVector3d : procedure(const v: TVector3d; s: TScalard; out res: TVector3d); register;
  ScaleHomVector3f : procedure(const v: TVector3f; s: TScalarf; out res: TVector3f); register;
  ScaleHomVector3d : procedure(const v: TVector3d; s: TScalard; out res: TVector3d); register;
  ScaleHomVector4f : procedure(const v: TVector4f; s: TScalarf; out res: TVector4f); register;
  ScaleHomVector4d : procedure(const v: TVector4d; s: TScalard; out res: TVector4d); register;

// *****************************************************************************
// CrossProductVector:
// *****************************************************************************

  CrossProductVector2f : procedure(const v1,v2f: TVector2f; out resf: TVector2f); register;
  CrossProductVector2d : procedure(const v1,v2d: TVector2d; out resd: TVector2d); register;
  CrossProductVector3f : procedure(const v1,v2f: TVector3f; out resf: TVector3f); register;
  CrossProductVector3d : procedure(const v1,v2d: TVector3d; out resd: TVector3d); register;
  CrossProductVector4f : procedure(const v1,v2f: TVector4f; out resf: TVector4f); register;
  CrossProductVector4d : procedure(const v1,v2d: TVector4d; out resd: TVector4d); register;
  CrossProductHomVector4f : procedure(const v1,v2: TVector4f; out res: TVector4f); register;
  CrossProductHomVector4d : procedure(const v1,v2: TVector4d; out res: TVector4d); register;

// *****************************************************************************
// DotProductVector:
// *****************************************************************************

  DotProductVector2f : function(const v1,v2: TVector2f) : TScalarf; register;
  DotProductVector2d : function(const v1,v2: TVector2d) : TScalard; register;
  DotProductVector3f : function(const v1,v2: TVector3f) : TScalarf; register;
  DotProductVector3d : function(const v1,v2: TVector3d) : TScalard; register;
  DotProductVector4f : function(const v1,v2: TVector4f) : TScalarf; register;
  DotProductVector4d : function(const v1,v2: TVector4d) : TScalard; register;
  DotProductHomVector3f : function(const v1,v2: TVector3f) : TScalarf; register;
  DotProductHomVector3d : function(const v1,v2: TVector3d) : TScalard; register;
  DotProductHomVector4f : function(const v1,v2: TVector4f) : TScalarf; register;
  DotProductHomVector4d : function(const v1,v2: TVector4d) : TScalard; register;

// *****************************************************************************
// SquareMagnitudeVector:
// *****************************************************************************

  SquareMagnitudeVector2f : function(const v: TVector2f) : TScalarf; register;
  SquareMagnitudeVector2d : function(const v: TVector2d) : TScalard; register;
  SquareMagnitudeVector3f : function(const v: TVector3f) : TScalarf; register;
  SquareMagnitudeVector3d : function(const v: TVector3d) : TScalard; register;
  SquareMagnitudeVector4f : function(const v: TVector4f) : TScalarf; register;
  SquareMagnitudeVector4d : function(const v: TVector4d) : TScalard; register;
  SquareMagnitudeHomVector3f : function(const v: TVector3f) : TScalarf; register;
  SquareMagnitudeHomVector3d : function(const v: TVector3d) : TScalard; register;
  SquareMagnitudeHomVector4f : function(const v: TVector4f) : TScalarf; register;
  SquareMagnitudeHomVector4d : function(const v: TVector4d) : TScalard; register;

// *****************************************************************************
// MagnitudeVector:
// *****************************************************************************

  MagnitudeVector2f : function(const v: TVector2f) : TScalarf; register;
  MagnitudeVector2d : function(const v: TVector2d) : TScalard; register;
  MagnitudeVector3f : function(const v: TVector3f) : TScalarf; register;
  MagnitudeVector3d : function(const v: TVector3d) : TScalard; register;
  MagnitudeVector4f : function(const v: TVector4f) : TScalarf; register;
  MagnitudeVector4d : function(const v: TVector4d) : TScalard; register;
  MagnitudeHomVector3f : function(const v: TVector3f) : TScalarf; register;
  MagnitudeHomVector3d : function(const v: TVector3d) : TScalard; register;
  MagnitudeHomVector4f : function(const v: TVector4f) : TScalarf; register;
  MagnitudeHomVector4d : function(const v: TVector4d) : TScalard; register;

// *****************************************************************************
// NormalizeVector:
// *****************************************************************************

  NormalizeVector2f : procedure(const v: TVector2f; out res: TVector2f); register;
  NormalizeVector2d : procedure(const v: TVector2d; out res: TVector2d); register;
  NormalizeVector3f : procedure(const v: TVector3f; out res: TVector3f); register;
  NormalizeVector3d : procedure(const v: TVector3d; out res: TVector3d); register;
  NormalizeVector4f : procedure(const v: TVector4f; out res: TVector4f); register;
  NormalizeVector4d : procedure(const v: TVector4d; out res: TVector4d); register;
  NormalizeHomVector3f : procedure(const v: TVector3f; out res: TVector3f); register;
  NormalizeHomVector3d : procedure(const v: TVector3d; out res: TVector3d); register;
  NormalizeHomVector4f : procedure(const v: TVector4f; out res: TVector4f); register;
  NormalizeHomVector4d : procedure(const v: TVector4d; out res: TVector4d); register;

// *****************************************************************************
// NegateVector:
// *****************************************************************************

  NegateVector2f : procedure(const v: TVector2f; out res: TVector2f); register;
  NegateVector2d : procedure(const v: TVector2d; out res: TVector2d); register;
  NegateVector3f : procedure(const v: TVector3f; out res: TVector3f); register;
  NegateVector3d : procedure(const v: TVector3d; out res: TVector3d); register;
  NegateVector4f : procedure(const v: TVector4f; out res: TVector4f); register;
  NegateVector4d : procedure(const v: TVector4d; out res: TVector4d); register;
  NegateHomVector3f : procedure(const v: TVector3f; out res: TVector3f); register;
  NegateHomVector3d : procedure(const v: TVector3d; out res: TVector3d); register;
  NegateHomVector4f : procedure(const v: TVector4f; out res: TVector4f); register;
  NegateHomVector4d : procedure(const v: TVector4d; out res: TVector4d); register;

// *****************************************************************************
// CombineVector:
// *****************************************************************************

  CombineVector2f : procedure(const v1,v2: TVector2f; f1,f2: TScalarf; out res: TVector2f); register;
  CombineVector2d : procedure(const v1,v2: TVector2d; f1,f2: TScalard; out res: TVector2d); register;
  CombineVector3f : procedure(const v1,v2: TVector3f; f1,f2: TScalarf; out res: TVector3f); register;
  CombineVector3d : procedure(const v1,v2: TVector3d; f1,f2: TScalard; out res: TVector3d); register;
  CombineVector4f : procedure(const v1,v2: TVector4f; f1,f2: TScalarf; out res: TVector4f); register;
  CombineVector4d : procedure(const v1,v2: TVector4d; f1,f2: TScalard; out res: TVector4d); register;

// *****************************************************************************
// LerpVector:
// *****************************************************************************

  LerpVector2f : procedure(const v1,v2: TVector2f; t: TScalarf; out res: TVector2f); register;
  LerpVector2d : procedure(const v1,v2: TVector2d; t: TScalard; out res: TVector2d); register;
  LerpVector3f : procedure(const v1,v2: TVector3f; t: TScalarf; out res: TVector3f); register;
  LerpVector3d : procedure(const v1,v2: TVector3d; t: TScalard; out res: TVector3d); register;
  LerpVector4f : procedure(const v1,v2: TVector4f; t: TScalarf; out res: TVector4f); register;
  LerpVector4d : procedure(const v1,v2: TVector4d; t: TScalard; out res: TVector4d); register;

// *****************************************************************************
// AngleVector:
// *****************************************************************************

  AngleVector2f : function(const v1,v2: TVector2f) : TAnglef; register;
  AngleVector2d : function(const v1,v2: TVector2d) : TAngled; register;
  AngleVector3f : function(const v1,v2: TVector3f) : TAnglef; register;
  AngleVector3d : function(const v1,v2: TVector3d) : TAngled; register;
  AngleVector4f : function(const v1,v2: TVector4f) : TAnglef; register;
  AngleVector4d : function(const v1,v2: TVector4d) : TAngled; register;
  AngleHomVector3f : function(const v1,v2: TVector3f) : TAnglef; register;
  AngleHomVector3d : function(const v1,v2: TVector3d) : TAngled; register;
  AngleHomVector4f : function(const v1,v2: TVector4f) : TAnglef; register;
  AngleHomVector4d : function(const v1,v2: TVector4d) : TAngled; register;

// *****************************************************************************
// PerpendicularVector:
// *****************************************************************************

  PerpendicularVector2f : procedure(const v  : TVector2f; out res: TVector2f); register;
  PerpendicularVector2d : procedure(const v  : TVector2d; out res: TVector2d); register;
  PerpendicularVector3f : procedure(const v,n: TVector3f; out res: TVector3f); register;
  PerpendicularVector3d : procedure(const v,n: TVector3d; out res: TVector3d); register;
  PerpendicularHomVector3f : procedure(const v  : TVector3f; out res: TVector3f); register;
  PerpendicularHomVector3d : procedure(const v  : TVector3d; out res: TVector3d); register;
  PerpendicularHomVector4f : procedure(const v,n: TVector4f; out res: TVector4f); register;
  PerpendicularHomVector4d : procedure(const v,n: TVector4d; out res: TVector4d); register;

// *****************************************************************************
// ReflectVector:
// *****************************************************************************

  ReflectVector2f : procedure(const v,n: TVector2f; out res: TVector2f); register;
  ReflectVector2d : procedure(const v,n: TVector2d; out res: TVector2d); register;
  ReflectVector3f : procedure(const v,n: TVector3f; out res: TVector3f); register;
  ReflectVector3d : procedure(const v,n: TVector3d; out res: TVector3d); register;
  ReflectVector4f : procedure(const v,n: TVector4f; out res: TVector4f); register;
  ReflectVector4d : procedure(const v,n: TVector4d; out res: TVector4d); register;
  ReflectHomVector3f : procedure(const v,n: TVector3f; out res: TVector3f); register;
  ReflectHomVector3d : procedure(const v,n: TVector3d; out res: TVector3d); register;
  ReflectHomVector4f : procedure(const v,n: TVector4f; out res: TVector4f); register;
  ReflectHomVector4d : procedure(const v,n: TVector4d; out res: TVector4d); register;

// *****************************************************************************
// MakeAffineVector:
// *****************************************************************************

  MakeAffineVector2f : procedure(const v: TVector3f; out res: TVector2f); register;
  MakeAffineVector2d : procedure(const v: TVector3d; out res: TVector2d); register;
  MakeAffineVector3f : procedure(const v: TVector4f; out res: TVector3f); register;
  MakeAffineVector3d : procedure(const v: TVector4d; out res: TVector3d); register;

// *****************************************************************************
// MakeHomogenVector:
// *****************************************************************************

  MakeHomogenVector3f : procedure(const v: TVector2f; out res: TVector3f); register;
  MakeHomogenVector3d : procedure(const v: TVector2d; out res: TVector3d); register;
  MakeHomogenVector4f : procedure(const v: TVector3f; out res: TVector4f); register;
  MakeHomogenVector4d : procedure(const v: TVector3d; out res: TVector4d); register;

//  Quaternions:

// *****************************************************************************
// Quaternion from angle / Quaternion to angle:
// *****************************************************************************

  QuaternionfFromAngle : procedure(const a: TAnglef; const v: TVector3f; out res: TQuaternionf); register;
  QuaterniondFromAngle : procedure(const a: TAngled; const v: TVector3d; out res: TQuaterniond); register;
  QuaternionfToAngle : procedure(const q: TQuaternionf; out a: TAnglef; out res: TVector3f); register;
  QuaterniondToAngle : procedure(const q: TQuaterniond; out a: TAngled; out res: TVector3d); register;

// *****************************************************************************
// NegateQuaternion
// *****************************************************************************

  NegateQuaternionf : procedure(const q: TQuaternionf; out res: TQuaternionf); register;
  NegateQuaterniond : procedure(const q: TQuaterniond; out res: TQuaterniond); register;

// *****************************************************************************
// InverseQuaternion
// *****************************************************************************

  InverseQuaternionf : procedure(const q: TQuaternionf; out res: TQuaternionf); register;
  InverseQuaterniond : procedure(const q: TQuaterniond; out res: TQuaterniond); register;

// *****************************************************************************
// Normalize Quaternion
// *****************************************************************************

  NormalizeQuaternionf : procedure(const q: TQuaternionf; out res: TQuaternionf); register;
  NormalizeQuaterniond : procedure(const q: TQuaterniond; out res: TQuaterniond); register;

// *****************************************************************************
// Add Quaternion
// *****************************************************************************

  AddQuaternionf : procedure(const q1, q2: TQuaternionf; out res: TQuaternionf); register;
  AddQuaterniond : procedure(const q1, q2: TQuaterniond; out res: TQuaterniond); register;

// *****************************************************************************
// Sub Quaternion
// *****************************************************************************

  SubQuaternionf : procedure(const q1, q2: TQuaternionf; out res: TQuaternionf); register;
  SubQuaterniond : procedure(const q1, q2: TQuaterniond; out res: TQuaterniond); register;

// *****************************************************************************
// Mul Quaternion
// *****************************************************************************

  MulQuaternionf : procedure(const q1, q2: TQuaternionf; out res: TQuaternionf); register;
  MulQuaterniond : procedure(const q1, q2: TQuaterniond; out res: TQuaterniond); register;

// *****************************************************************************
// Mul Quaternion
// *****************************************************************************

  DivQuaternionf : procedure(const q1, q2: TQuaternionf; out res: TQuaternionf); register;
  DivQuaterniond : procedure(const q1, q2: TQuaterniond; out res: TQuaterniond); register;

// *****************************************************************************
// Square Quaternion
// *****************************************************************************

  SquareQuaternionf : procedure(const q: TQuaternionf; out res: TQuaternionf); register;
  SquareQuaterniond : procedure(const q: TQuaterniond; out res: TQuaterniond); register;

// *****************************************************************************
// Sqrt Quaternion
// *****************************************************************************

  SqrtQuaternionf : procedure(const q: TQuaternionf; out res: TQuaternionf); register;
  SqrtQuaterniond : procedure(const q: TQuaterniond; out res: TQuaterniond); register;

// *****************************************************************************
// MagnitudeQuaternion
// *****************************************************************************

  MagnitudeQuaternionf : function(const q: TQuaternionf): TScalarf; register;
  MagnitudeQuaterniond : function(const q: TQuaterniond): TScalard; register;

// *****************************************************************************
// DotProductQuaternion
// *****************************************************************************

  DotProductQuaternionf : function(const q1, q2: TQuaternionf): TScalarf; register;
  DotProductQuaterniond : function(const q1, q2: TQuaterniond): TScalard; register;
  DotProductUnitQuaternionf : function(const q1, q2: TQuaternionf): TScalarf; register;
  DotProductUnitQuaterniond : function(const q1, q2: TQuaterniond): TScalard; register;

// *****************************************************************************
// LogQuaternionf
// *****************************************************************************

  LogQuaternionf : procedure(const q: TQuaternionf; out res: TQuaternionf); register;
  LogQuaterniond : procedure(const q: TQuaterniond; out res: TQuaterniond); register;
  ExpQuaternionf : procedure(const q: TQuaternionf; out res: TQuaternionf); register;
  ExpQuaterniond : procedure(const q: TQuaterniond; out res: TQuaterniond); register;
  LnQuaternionf  : procedure(const q1, q2: TQuaternionf; out res: TQuaternionf); register;
  LnQuaterniond  : procedure(const q1, q2: TQuaterniond; out res: TQuaterniond); register;

// *****************************************************************************
// SphericalInterpolationQuaternion
// *****************************************************************************

  SlerpQuaternionf : procedure(const q1, q2: TQuaternionf; const Spin, Alpha: TAnglef; out res: TQuaternionf);
  SlerpQuaterniond : procedure(const q1, q2: TQuaterniond; const Spin, Alpha: TAngled; out res: TQuaterniond);
  SlerplQuaternionf : procedure(const q1, q2: TQuaternionf; const Spin, Alpha: TAnglef; out res: TQuaternionf);
  SlerplQuaterniond : procedure(const q1, q2: TQuaterniond; const Spin, Alpha: TAngled; out res: TQuaterniond);

// *****************************************************************************
// QuaternionToMatrix
// *****************************************************************************

  QuaternionToMatrix4f: procedure(const q: TQuaternionf; out res: TMatrix4f);
  QuaternionToMatrix4d: procedure(const q: TQuaterniond; out res: TMatrix4d);

//  Matrices:

// *****************************************************************************
// MulMatrix:
// *****************************************************************************

  MulMatrix2f : procedure(const m1,m2: TMatrix2f; out res: TMatrix2f); register;
  MulMatrix2d : procedure(const m1,m2: TMatrix2d; out res: TMatrix2d); register;
  MulMatrix3f : procedure(const m1,m2: TMatrix3f; out res: TMatrix3f); register;
  MulMatrix3d : procedure(const m1,m2: TMatrix3d; out res: TMatrix3d); register;
  MulMatrix4f : procedure(const m1,m2: TMatrix4f; out res: TMatrix4f); register;
  MulMatrix4d : procedure(const m1,m2: TMatrix4d; out res: TMatrix4d); register;

  TranspMulMatrix2f : procedure(const m1,m2: TMatrix2f; out res: TMatrix2f); register;
  TranspMulMatrix2d : procedure(const m1,m2: TMatrix2d; out res: TMatrix2d); register;
  TranspMulMatrix3f : procedure(const m1,m2: TMatrix3f; out res: TMatrix3f); register;
  TranspMulMatrix3d : procedure(const m1,m2: TMatrix3d; out res: TMatrix3d); register;
  TranspMulMatrix4f : procedure(const m1,m2: TMatrix4f; out res: TMatrix4f); register;
  TranspMulMatrix4d : procedure(const m1,m2: TMatrix4d; out res: TMatrix4d); register;

// *****************************************************************************
// TransposeMatrix:
// *****************************************************************************

  TransposeMatrix2f : procedure(const m: TMatrix2f; out res: TMatrix2f); register;
  TransposeMatrix2d : procedure(const m: TMatrix2d; out res: TMatrix2d); register;
  TransposeMatrix3f : procedure(const m: TMatrix3f; out res: TMatrix3f); register;
  TransposeMatrix3d : procedure(const m: TMatrix3d; out res: TMatrix3d); register;
  TransposeMatrix4f : procedure(const m: TMatrix4f; out res: TMatrix4f); register;
  TransposeMatrix4d : procedure(const m: TMatrix4d; out res: TMatrix4d); register;

// *****************************************************************************
// ScaleMatrix:
// *****************************************************************************

  ScaleMatrix3f : procedure(x,y   : TScalarf; out res: TMatrix3f); register;
  ScaleMatrix3d : procedure(x,y   : TScalard; out res: TMatrix3d); register;
  ScaleMatrix4f : procedure(x,y,z : TScalarf; out res: TMatrix4f); register;
  ScaleMatrix4d : procedure(x,y,z : TScalard; out res: TMatrix4d); register;

  MulScalarMatrix3f : procedure(const m: TMatrix3f; s: TScalarf; out res: TMatrix3f); register;
  MulScalarMatrix3d : procedure(const m: TMatrix3d; s: TScalard; out res: TMatrix3d); register;
  MulScalarMatrix4f : procedure(const m: TMatrix4f; s: TScalarf; out res: TMatrix4f); register;
  MulScalarMatrix4d : procedure(const m: TMatrix4d; s: TScalard; out res: TMatrix4d); register;

// *****************************************************************************
// TranslationMatrix:
// *****************************************************************************

  TranslationMatrix3f : procedure(const d : TVector2f; out res: TMatrix3f); register;
  TranslationMatrix3d : procedure(const d : TVector2d; out res: TMatrix3d); register;
  TranslationMatrix4f : procedure(const d : TVector3f; out res: TMatrix4f); register;
  TranslationMatrix4d : procedure(const d : TVector3d; out res: TMatrix4d); register;

  XYTranslationMatrix3f  : procedure(x,y : TScalarf; out res: TMatrix3f); register;
  XYTranslationMatrix3d  : procedure(x,y : TScalard; out res: TMatrix3d); register;
  XYZTranslationMatrix4f : procedure(x,y,z : TScalarf; out res: TMatrix4f); register;
  XYZTranslationMatrix4d : procedure(x,y,z : TScalard; out res: TMatrix4d); register;

// *****************************************************************************
// RotationMatrix:
// *****************************************************************************

  XRotationMatrix4f : procedure(a: TAnglef; out res: TMatrix4f); register;
  XRotationMatrix4d : procedure(a: TAngled; out res: TMatrix4d); register;
  YRotationMatrix4f : procedure(a: TAnglef; out res: TMatrix4f); register;
  YRotationMatrix4d : procedure(a: TAngled; out res: TMatrix4d); register;
  ZRotationMatrix4f : procedure(a: TAnglef; out res: TMatrix4f); register;
  ZRotationMatrix4d : procedure(a: TAngled; out res: TMatrix4d); register;

  RotationMatrix3f : procedure(a: TAnglef; p: TPoint2f; out res: TMatrix3f); register;
  RotationMatrix3d : procedure(a: TAngled; p: TPoint2d; out res: TMatrix3d); register;
  RotationMatrix4f : procedure(a: TAnglef; v: TVector3f; out res: TMatrix4f); register;
  RotationMatrix4d : procedure(a: TAngled; v: TVector3d; out res: TMatrix4d); register;

  XYRotationMatrix3f  : procedure(a: TAnglef; x,y: TScalarf; out res: TMatrix3f); register;
  XYRotationMatrix3d  : procedure(a: TAngled; x,y: TScalard; out res: TMatrix3d); register;
  XYZRotationMatrix4f : procedure(a: TAnglef; x,y,z: TScalarf; out res: TMatrix4f); register;
  XYZRotationMatrix4d : procedure(a: TAngled; x,y,z: TScalard; out res: TMatrix4d); register;

// *****************************************************************************
// AdjustMatrix:
// *****************************************************************************

  AdjustAxisMatrix3f : procedure(const x: TVector2f; out res: TMatrix3f); register;
  AdjustAxisMatrix3d : procedure(const x: TVector2d; out res: TMatrix3d); register;
  AdjustAxisMatrix4f : procedure(const x,y: TVector3f; out res: TMatrix4f); register;
  AdjustAxisMatrix4d : procedure(const x,y: TVector3d; out res: TMatrix4d); register;

// *****************************************************************************
// DeterminantMatrix:
// *****************************************************************************

  DeterminantMatrix3f: function(const m: TMatrix3f): TScalarf; register;
  DeterminantMatrix3d: function(const m: TMatrix3d): TScalard; register;
  DeterminantMatrix4f: function(const m: TMatrix4f): TScalarf; register;
  DeterminantMatrix4d: function(const m: TMatrix4d): TScalard; register;

// *****************************************************************************
// AdjointMatrix:
// *****************************************************************************

  AdjointMatrix4f: procedure(const m: TMatrix4f; out res: TMatrix4f); register;
  AdjointMatrix4d: procedure(const m: TMatrix4d; out res: TMatrix4d); register;

// *****************************************************************************
// InverseMatrix:
// *****************************************************************************

  InverseMatrix3f : procedure(const m: TMatrix3f; out res: TMatrix3f); register;
  InverseMatrix3d : procedure(const m: TMatrix3d; out res: TMatrix3d); register;
  InverseMatrix4f : procedure(const m: TMatrix4f; out res: TMatrix4f); register;
  InverseMatrix4d : procedure(const m: TMatrix4d; out res: TMatrix4d); register;

//  Transformations:

// *****************************************************************************
// TransformVector:
// *****************************************************************************

  TransformVector3f : procedure(const v: TVector3f; const m: TMatrix3f; out res: TVector3f); register;
  TransformVector3d : procedure(const v: TVector3d; const m: TMatrix3d; out res: TVector3d); register;
  TransformVector4f : procedure(const v: TVector4f; const m: TMatrix4f; out res: TVector4f); register;
  TransformVector4d : procedure(const v: TVector4d; const m: TMatrix4d; out res: TVector4d); register;

  M4fTransformVector3f: procedure(const v: TVector3f; const m: TMatrix4f; out res: TVector3f); register;
  M4dTransformVector3d: procedure(const v: TVector3d; const m: TMatrix4d; out res: TVector3d); register;

  TranspTransformVector3f : procedure(const v: TVector3f; const m: TMatrix3f; out res: TVector3f); register;
  TranspTransformVector3d : procedure(const v: TVector3d; const m: TMatrix3d; out res: TVector3d); register;
  TranspTransformVector4f : procedure(const v: TVector4f; const m: TMatrix4f; out res: TVector4f); register;
  TranspTransformVector4d : procedure(const v: TVector4d; const m: TMatrix4d; out res: TVector4d); register;

  M4fTranspTransformVector3f: procedure(const v: TVector3f; const m: TMatrix4f; out res: TVector3f); register;
  M4dTranspTransformVector3d: procedure(const v: TVector3d; const m: TMatrix4d; out res: TVector3d); register;


// *****************************************************************************
// *****************************************************************************
implementation
// *****************************************************************************
// *****************************************************************************


uses
  SysUtils,
  X86Ext,
  GeometryFunctionsX87Pascal,
  GeometryFunctionsX87,
  GeometryFunctions3DNow;

procedure NI; // Not Implemented
begin
  raise Exception.Create('Function not implemented');
end;

// *****************************************************************************
// Internal:
// *****************************************************************************

var
  GeometryFunctionsMode : TGeometryFunctionsMode;

procedure SetX87PascalGeometryFunctionsMode;
begin

//  Vector Algebra:

// *****************************************************************************
// AddVector:
// *****************************************************************************

  AddVector2f := GeometryFunctionsX87Pascal.AddVector2f;
  AddVector2d := GeometryFunctionsX87Pascal.AddVector2d;
  AddVector3f := GeometryFunctionsX87Pascal.AddVector3f;
  AddVector3d := GeometryFunctionsX87Pascal.AddVector3d;
  AddVector4f := GeometryFunctionsX87Pascal.AddVector4f;
  AddVector4d := GeometryFunctionsX87Pascal.AddVector4d;
  AddHomVector3f := GeometryFunctionsX87Pascal.AddHomVector3f;
  AddHomVector3d := GeometryFunctionsX87Pascal.AddHomVector3d;
  AddHomVector4f := GeometryFunctionsX87Pascal.AddHomVector4f;
  AddHomVector4d := GeometryFunctionsX87Pascal.AddHomVector4d;

// *****************************************************************************
// SubVector:
// *****************************************************************************

  SubVector2f := GeometryFunctionsX87Pascal.SubVector2f;
  SubVector2d := GeometryFunctionsX87Pascal.SubVector2d;
  SubVector3f := GeometryFunctionsX87Pascal.SubVector3f;
  SubVector3d := GeometryFunctionsX87Pascal.SubVector3d;
  SubVector4f := GeometryFunctionsX87Pascal.SubVector4f;
  SubVector4d := GeometryFunctionsX87Pascal.SubVector4d;
  SubHomVector3f := GeometryFunctionsX87Pascal.SubHomVector3f;
  SubHomVector3d := GeometryFunctionsX87Pascal.SubHomVector3d;
  SubHomVector4f := GeometryFunctionsX87Pascal.SubHomVector4f;
  SubHomVector4d := GeometryFunctionsX87Pascal.SubHomVector4d;

// *****************************************************************************
// AddScalarVector:
// *****************************************************************************

  AddScalarVector2f := GeometryFunctionsX87Pascal.AddScalarVector2f;
  AddScalarVector2d := GeometryFunctionsX87Pascal.AddScalarVector2d;
  AddScalarVector3f := GeometryFunctionsX87Pascal.AddScalarVector3f;
  AddScalarVector3d := GeometryFunctionsX87Pascal.AddScalarVector3d;
  AddScalarVector4f := GeometryFunctionsX87Pascal.AddScalarVector4f;
  AddScalarVector4d := GeometryFunctionsX87Pascal.AddScalarVector4d;

// *****************************************************************************
// SubScalarVector:
// *****************************************************************************

  SubScalarVector2f := GeometryFunctionsX87Pascal.SubScalarVector2f;
  SubScalarVector2d := GeometryFunctionsX87Pascal.SubScalarVector2d;
  SubScalarVector3f := GeometryFunctionsX87Pascal.SubScalarVector3f;
  SubScalarVector3d := GeometryFunctionsX87Pascal.SubScalarVector3d;
  SubScalarVector4f := GeometryFunctionsX87Pascal.SubScalarVector4f;
  SubScalarVector4d := GeometryFunctionsX87Pascal.SubScalarVector4d;

// *****************************************************************************
// MulVector:
// *****************************************************************************

  MulVector2f := GeometryFunctionsX87Pascal.MulVector2f;
  MulVector2d := GeometryFunctionsX87Pascal.MulVector2d;
  MulVector3f := GeometryFunctionsX87Pascal.MulVector3f;
  MulVector3d := GeometryFunctionsX87Pascal.MulVector3d;
  MulVector4f := GeometryFunctionsX87Pascal.MulVector4f;
  MulVector4d := GeometryFunctionsX87Pascal.MulVector4d;

// *****************************************************************************
// DivVector:
// *****************************************************************************

  DivVector2f := GeometryFunctionsX87Pascal.DivVector2f;
  DivVector2d := GeometryFunctionsX87Pascal.DivVector2d;
  DivVector3f := GeometryFunctionsX87Pascal.DivVector3f;
  DivVector3d := GeometryFunctionsX87Pascal.DivVector3d;
  DivVector4f := GeometryFunctionsX87Pascal.DivVector4f;
  DivVector4d := GeometryFunctionsX87Pascal.DivVector4d;

// *****************************************************************************
// MulScalarVector:
// *****************************************************************************

  MulScalarVector2f := GeometryFunctionsX87Pascal.MulScalarVector2f;
  MulScalarVector2d := GeometryFunctionsX87Pascal.MulScalarVector2d;
  MulScalarVector3f := GeometryFunctionsX87Pascal.MulScalarVector3f;
  MulScalarVector3d := GeometryFunctionsX87Pascal.MulScalarVector3d;
  MulScalarVector4f := GeometryFunctionsX87Pascal.MulScalarVector4f;
  MulScalarVector4d := GeometryFunctionsX87Pascal.MulScalarVector4d;

// *****************************************************************************
// DivScalarVector:
// *****************************************************************************

  DivScalarVector2f := GeometryFunctionsX87Pascal.DivScalarVector2f;
  DivScalarVector2d := GeometryFunctionsX87Pascal.DivScalarVector2d;
  DivScalarVector3f := GeometryFunctionsX87Pascal.DivScalarVector3f;
  DivScalarVector3d := GeometryFunctionsX87Pascal.DivScalarVector3d;
  DivScalarVector4f := GeometryFunctionsX87Pascal.DivScalarVector4f;
  DivScalarVector4d := GeometryFunctionsX87Pascal.DivScalarVector4d;

// *****************************************************************************
// ScaleVector:
// *****************************************************************************

  ScaleVector2f  := GeometryFunctionsX87Pascal.ScaleVector2f;
  ScaleVector2d  := GeometryFunctionsX87Pascal.ScaleVector2d;
  ScaleVector3f  := GeometryFunctionsX87Pascal.ScaleVector3f;
  ScaleVector3d  := GeometryFunctionsX87Pascal.ScaleVector3d;
  ScaleHomVector3f := GeometryFunctionsX87Pascal.ScaleHomVector3f;
  ScaleHomVector3d := GeometryFunctionsX87Pascal.ScaleHomVector3d;
  ScaleHomVector4f := GeometryFunctionsX87Pascal.ScaleHomVector4f;
  ScaleHomVector4d := GeometryFunctionsX87Pascal.ScaleHomVector4d;

// *****************************************************************************
// CrossProductVector:
// *****************************************************************************

  CrossProductVector2f := GeometryFunctionsX87Pascal.CrossProductVector2f;
  CrossProductVector2d := GeometryFunctionsX87Pascal.CrossProductVector2d;
  CrossProductVector3f := GeometryFunctionsX87Pascal.CrossProductVector3f;
  CrossProductVector3d := GeometryFunctionsX87Pascal.CrossProductVector3d;
  CrossProductVector4f := GeometryFunctionsX87Pascal.CrossProductVector4f;
  CrossProductVector4d := GeometryFunctionsX87Pascal.CrossProductVector4d;
  CrossProductHomVector4f := GeometryFunctionsX87Pascal.CrossProductHomVector4f;
  CrossProductHomVector4d := GeometryFunctionsX87Pascal.CrossProductHomVector4d;

// *****************************************************************************
// DotProductVector:
// *****************************************************************************

  DotProductVector2f := GeometryFunctionsX87Pascal.DotProductVector2f;
  DotProductVector2d := GeometryFunctionsX87Pascal.DotProductVector2d;
  DotProductVector3f := GeometryFunctionsX87Pascal.DotProductVector3f;
  DotProductVector3d := GeometryFunctionsX87Pascal.DotProductVector3d;
  DotProductVector4f := GeometryFunctionsX87Pascal.DotProductVector4f;
  DotProductVector4d := GeometryFunctionsX87Pascal.DotProductVector4d;
  DotProductHomVector3f := GeometryFunctionsX87Pascal.DotProductHomVector3f;
  DotProductHomVector3d := GeometryFunctionsX87Pascal.DotProductHomVector3d;
  DotProductHomVector4f := GeometryFunctionsX87Pascal.DotProductHomVector4f;
  DotProductHomVector4d := GeometryFunctionsX87Pascal.DotProductHomVector4d;

// *****************************************************************************
// SquareMagnitudeVector:
// *****************************************************************************

  SquareMagnitudeVector2f := GeometryFunctionsX87Pascal.SquareMagnitudeVector2f;
  SquareMagnitudeVector2d := GeometryFunctionsX87Pascal.SquareMagnitudeVector2d;
  SquareMagnitudeVector3f := GeometryFunctionsX87Pascal.SquareMagnitudeVector3f;
  SquareMagnitudeVector3d := GeometryFunctionsX87Pascal.SquareMagnitudeVector3d;
  SquareMagnitudeVector4f := GeometryFunctionsX87Pascal.SquareMagnitudeVector4f;
  SquareMagnitudeVector4d := GeometryFunctionsX87Pascal.SquareMagnitudeVector4d;
  SquareMagnitudeHomVector3f := GeometryFunctionsX87Pascal.SquareMagnitudeHomVector3f;
  SquareMagnitudeHomVector3d := GeometryFunctionsX87Pascal.SquareMagnitudeHomVector3d;
  SquareMagnitudeHomVector4f := GeometryFunctionsX87Pascal.SquareMagnitudeHomVector4f;
  SquareMagnitudeHomVector4d := GeometryFunctionsX87Pascal.SquareMagnitudeHomVector4d;

// *****************************************************************************
// MagnitudeVector:
// *****************************************************************************

  MagnitudeVector2f := GeometryFunctionsX87Pascal.MagnitudeVector2f;
  MagnitudeVector2d := GeometryFunctionsX87Pascal.MagnitudeVector2d;
  MagnitudeVector3f := GeometryFunctionsX87Pascal.MagnitudeVector3f;
  MagnitudeVector3d := GeometryFunctionsX87Pascal.MagnitudeVector3d;
  MagnitudeVector4f := GeometryFunctionsX87Pascal.MagnitudeVector4f;
  MagnitudeVector4d := GeometryFunctionsX87Pascal.MagnitudeVector4d;
  MagnitudeHomVector3f := GeometryFunctionsX87Pascal.MagnitudeHomVector3f;
  MagnitudeHomVector3d := GeometryFunctionsX87Pascal.MagnitudeHomVector3d;
  MagnitudeHomVector4f := GeometryFunctionsX87Pascal.MagnitudeHomVector4f;
  MagnitudeHomVector4d := GeometryFunctionsX87Pascal.MagnitudeHomVector4d;

// *****************************************************************************
// NormalizeVector:
// *****************************************************************************

  NormalizeVector2f := GeometryFunctionsX87Pascal.NormalizeVector2f;
  NormalizeVector2d := GeometryFunctionsX87Pascal.NormalizeVector2d;
  NormalizeVector3f := GeometryFunctionsX87Pascal.NormalizeVector3f;
  NormalizeVector3d := GeometryFunctionsX87Pascal.NormalizeVector3d;
  NormalizeVector4f := GeometryFunctionsX87Pascal.NormalizeVector4f;
  NormalizeVector4d := GeometryFunctionsX87Pascal.NormalizeVector4d;
  NormalizeHomVector3f := GeometryFunctionsX87Pascal.NormalizeHomVector3f;
  NormalizeHomVector3d := GeometryFunctionsX87Pascal.NormalizeHomVector3d;
  NormalizeHomVector4f := GeometryFunctionsX87Pascal.NormalizeHomVector4f;
  NormalizeHomVector4d := GeometryFunctionsX87Pascal.NormalizeHomVector4d;

// *****************************************************************************
// NegateVector:
// *****************************************************************************

  NegateVector2f := GeometryFunctionsX87Pascal.NegateVector2f;
  NegateVector2d := GeometryFunctionsX87Pascal.NegateVector2d;
  NegateVector3f := GeometryFunctionsX87Pascal.NegateVector3f;
  NegateVector3d := GeometryFunctionsX87Pascal.NegateVector3d;
  NegateVector4f := GeometryFunctionsX87Pascal.NegateVector4f;
  NegateVector4d := GeometryFunctionsX87Pascal.NegateVector4d;
  NegateHomVector3f := GeometryFunctionsX87Pascal.NegateHomVector3f;
  NegateHomVector3d := GeometryFunctionsX87Pascal.NegateHomVector3d;
  NegateHomVector4f := GeometryFunctionsX87Pascal.NegateHomVector4f;
  NegateHomVector4d := GeometryFunctionsX87Pascal.NegateHomVector4d;

// *****************************************************************************
// CombineVector:
// *****************************************************************************

  CombineVector2f := GeometryFunctionsX87Pascal.CombineVector2f;
  CombineVector2d := GeometryFunctionsX87Pascal.CombineVector2d;
  CombineVector3f := GeometryFunctionsX87Pascal.CombineVector3f;
  CombineVector3d := GeometryFunctionsX87Pascal.CombineVector3d;
  CombineVector4f := GeometryFunctionsX87Pascal.CombineVector4f;
  CombineVector4d := GeometryFunctionsX87Pascal.CombineVector4d;

// *****************************************************************************
// LerpVector:
// *****************************************************************************

  LerpVector2f := GeometryFunctionsX87Pascal.LerpVector2f;
  LerpVector2d := GeometryFunctionsX87Pascal.LerpVector2d;
  LerpVector3f := GeometryFunctionsX87Pascal.LerpVector3f;
  LerpVector3d := GeometryFunctionsX87Pascal.LerpVector3d;
  LerpVector4f := GeometryFunctionsX87Pascal.LerpVector4f;
  LerpVector4d := GeometryFunctionsX87Pascal.LerpVector4d;

// *****************************************************************************
// AngleVector:
// *****************************************************************************

  AngleVector2f  := GeometryFunctionsX87Pascal.AngleVector2f;
  AngleVector2d  := GeometryFunctionsX87Pascal.AngleVector2d;
  AngleVector3f  := GeometryFunctionsX87Pascal.AngleVector3f;
  AngleVector3d  := GeometryFunctionsX87Pascal.AngleVector3d;
  AngleVector4f := GeometryFunctionsX87Pascal.AngleVector4f;
  AngleVector4d := GeometryFunctionsX87Pascal.AngleVector4d;
  AngleHomVector3f := GeometryFunctionsX87Pascal.AngleHomVector3f;
  AngleHomVector3d := GeometryFunctionsX87Pascal.AngleHomVector3d;
  AngleHomVector4f := GeometryFunctionsX87Pascal.AngleHomVector4f;
  AngleHomVector4d := GeometryFunctionsX87Pascal.AngleHomVector4d;

// *****************************************************************************
// PerpendicularVector:
// *****************************************************************************

  PerpendicularVector2f  := GeometryFunctionsX87Pascal.PerpendicularVector2f;
  PerpendicularVector2d  := GeometryFunctionsX87Pascal.PerpendicularVector2d;
  PerpendicularVector3f  := GeometryFunctionsX87Pascal.PerpendicularVector3f;
  PerpendicularVector3d  := GeometryFunctionsX87Pascal.PerpendicularVector3d;
  PerpendicularHomVector3f := GeometryFunctionsX87Pascal.PerpendicularHomVector3f;
  PerpendicularHomVector3d := GeometryFunctionsX87Pascal.PerpendicularHomVector3d;
  PerpendicularHomVector4f := GeometryFunctionsX87Pascal.PerpendicularHomVector4f;
  PerpendicularHomVector4d := GeometryFunctionsX87Pascal.PerpendicularHomVector4d;

// *****************************************************************************
// ReflectVector:
// *****************************************************************************

  ReflectVector2f  := GeometryFunctionsX87Pascal.ReflectVector2f;
  ReflectVector2d  := GeometryFunctionsX87Pascal.ReflectVector2d;
  ReflectVector3f  := GeometryFunctionsX87Pascal.ReflectVector3f;
  ReflectVector3d  := GeometryFunctionsX87Pascal.ReflectVector3d;
  ReflectVector4f := GeometryFunctionsX87Pascal.ReflectVector4f;
  ReflectVector4d := GeometryFunctionsX87Pascal.ReflectVector4d;
  ReflectHomVector3f := GeometryFunctionsX87Pascal.ReflectHomVector3f;
  ReflectHomVector3d := GeometryFunctionsX87Pascal.ReflectHomVector3d;
  ReflectHomVector4f := GeometryFunctionsX87Pascal.ReflectHomVector4f;
  ReflectHomVector4d := GeometryFunctionsX87Pascal.ReflectHomVector4d;

// *****************************************************************************
// MakeAffineVector:
// *****************************************************************************

  MakeAffineVector2f := GeometryFunctionsX87Pascal.MakeAffineVector2f;
  MakeAffineVector2d := GeometryFunctionsX87Pascal.MakeAffineVector2d;
  MakeAffineVector3f := GeometryFunctionsX87Pascal.MakeAffineVector3f;
  MakeAffineVector3d := GeometryFunctionsX87Pascal.MakeAffineVector3d;

// *****************************************************************************
// MakeHomogenVector:
// *****************************************************************************

  MakeHomogenVector3f := GeometryFunctionsX87Pascal.MakeHomogenVector3f;
  MakeHomogenVector3d := GeometryFunctionsX87Pascal.MakeHomogenVector3d;
  MakeHomogenVector4f := GeometryFunctionsX87Pascal.MakeHomogenVector4f;
  MakeHomogenVector4d := GeometryFunctionsX87Pascal.MakeHomogenVector4d;

//  Quaternions:

// *****************************************************************************
// Quaternion from angle / Quaternion to angle:
// *****************************************************************************

  QuaternionfFromAngle := GeometryFunctionsX87Pascal.QuaternionfFromAngle;
  QuaterniondFromAngle := GeometryFunctionsX87Pascal.QuaterniondFromAngle;
  QuaternionfToAngle := GeometryFunctionsX87Pascal.QuaternionfToAngle;
  QuaterniondToAngle := GeometryFunctionsX87Pascal.QuaterniondToAngle;

// *****************************************************************************
// NegateQuaternion
// *****************************************************************************

  NegateQuaternionf := GeometryFunctionsX87Pascal.NegateQuaternionf;
  NegateQuaterniond := GeometryFunctionsX87Pascal.NegateQuaterniond;

// *****************************************************************************
// InverseQuaternion
// *****************************************************************************

  InverseQuaternionf := GeometryFunctionsX87Pascal.InverseQuaternionf;
  InverseQuaterniond := GeometryFunctionsX87Pascal.InverseQuaterniond;

// *****************************************************************************
// Normalize Quaternion
// *****************************************************************************

  NormalizeQuaternionf := GeometryFunctionsX87Pascal.NormalizeQuaternionf;
  NormalizeQuaterniond := GeometryFunctionsX87Pascal.NormalizeQuaterniond;

// *****************************************************************************
// Add Quaternion
// *****************************************************************************

  AddQuaternionf := GeometryFunctionsX87Pascal.AddQuaternionf;
  AddQuaterniond := GeometryFunctionsX87Pascal.AddQuaterniond;

// *****************************************************************************
// Sub Quaternion
// *****************************************************************************

  SubQuaternionf := GeometryFunctionsX87Pascal.SubQuaternionf;
  SubQuaterniond := GeometryFunctionsX87Pascal.SubQuaterniond;

// *****************************************************************************
// Mul Quaternion
// *****************************************************************************

  MulQuaternionf := GeometryFunctionsX87Pascal.MulQuaternionf;
  MulQuaterniond := GeometryFunctionsX87Pascal.MulQuaterniond;

// *****************************************************************************
// Mul Quaternion
// *****************************************************************************

  DivQuaternionf := GeometryFunctionsX87Pascal.DivQuaternionf;
  DivQuaterniond := GeometryFunctionsX87Pascal.DivQuaterniond;

// *****************************************************************************
// Square Quaternion
// *****************************************************************************

  SquareQuaternionf := GeometryFunctionsX87Pascal.SquareQuaternionf;
  SquareQuaterniond := GeometryFunctionsX87Pascal.SquareQuaterniond;

// *****************************************************************************
// Sqrt Quaternion
// *****************************************************************************

  SqrtQuaternionf := GeometryFunctionsX87Pascal.SqrtQuaternionf;
  SqrtQuaterniond := GeometryFunctionsX87Pascal.SqrtQuaterniond;

// *****************************************************************************
// MagnitudeQuaternion
// *****************************************************************************

  MagnitudeQuaternionf := GeometryFunctionsX87Pascal.MagnitudeQuaternionf;
  MagnitudeQuaterniond := GeometryFunctionsX87Pascal.MagnitudeQuaterniond;

// *****************************************************************************
// DotProductQuaternion
// *****************************************************************************

  DotProductQuaternionf := GeometryFunctionsX87Pascal.DotProductQuaternionf;
  DotProductQuaterniond := GeometryFunctionsX87Pascal.DotProductQuaterniond;
  DotProductUnitQuaternionf := GeometryFunctionsX87Pascal.DotProductUnitQuaternionf;
  DotProductUnitQuaterniond := GeometryFunctionsX87Pascal.DotProductUnitQuaterniond;

// *****************************************************************************
// LogQuaternionf
// *****************************************************************************

  LogQuaternionf := GeometryFunctionsX87Pascal.LogQuaternionf;
  LogQuaterniond := GeometryFunctionsX87Pascal.LogQuaterniond;
  ExpQuaternionf := GeometryFunctionsX87Pascal.ExpQuaternionf;
  ExpQuaterniond := GeometryFunctionsX87Pascal.ExpQuaterniond;
  LnQuaternionf  := GeometryFunctionsX87Pascal.LnQuaternionf;
  LnQuaterniond  := GeometryFunctionsX87Pascal.LnQuaterniond;

// *****************************************************************************
// SphericalInterpolationQuaternion
// *****************************************************************************

  SlerpQuaternionf := GeometryFunctionsX87Pascal.SlerpQuaternionf;
  SlerpQuaterniond := GeometryFunctionsX87Pascal.SlerpQuaterniond;
  SlerplQuaternionf := GeometryFunctionsX87Pascal.SlerplQuaternionf;
  SlerplQuaterniond := GeometryFunctionsX87Pascal.SlerplQuaterniond;

// *****************************************************************************
// QuaternionToMatrix
// *****************************************************************************

  QuaternionToMatrix4f := GeometryFunctionsX87Pascal.QuaternionToMatrix4f;
  QuaternionToMatrix4d := GeometryFunctionsX87Pascal.QuaternionToMatrix4d;

//  Matrices:

// *****************************************************************************
// MulMatrix:
// *****************************************************************************

  MulMatrix2f := GeometryFunctionsX87Pascal.MulMatrix2f;
  MulMatrix2d := GeometryFunctionsX87Pascal.MulMatrix2d;
  MulMatrix3f := GeometryFunctionsX87Pascal.MulMatrix3f;
  MulMatrix3d := GeometryFunctionsX87Pascal.MulMatrix3d;
  MulMatrix4f := GeometryFunctionsX87Pascal.MulMatrix4f;
  MulMatrix4d := GeometryFunctionsX87Pascal.MulMatrix4d;

  TranspMulMatrix2f := GeometryFunctionsX87Pascal.TranspMulMatrix2f;
  TranspMulMatrix2d := GeometryFunctionsX87Pascal.TranspMulMatrix2d;
  TranspMulMatrix3f := GeometryFunctionsX87Pascal.TranspMulMatrix3f;
  TranspMulMatrix3d := GeometryFunctionsX87Pascal.TranspMulMatrix3d;
  TranspMulMatrix4f := GeometryFunctionsX87Pascal.TranspMulMatrix4f;
  TranspMulMatrix4d := GeometryFunctionsX87Pascal.TranspMulMatrix4d;

// *****************************************************************************
// TransposeMatrix:
// *****************************************************************************

  TransposeMatrix2f := GeometryFunctionsX87Pascal.TransposeMatrix2f;
  TransposeMatrix2d := GeometryFunctionsX87Pascal.TransposeMatrix2d;
  TransposeMatrix3f := GeometryFunctionsX87Pascal.TransposeMatrix3f;
  TransposeMatrix3d := GeometryFunctionsX87Pascal.TransposeMatrix3d;
  TransposeMatrix4f := GeometryFunctionsX87Pascal.TransposeMatrix4f;
  TransposeMatrix4d := GeometryFunctionsX87Pascal.TransposeMatrix4d;

// *****************************************************************************
// ScaleMatrix:
// *****************************************************************************

  ScaleMatrix3f := GeometryFunctionsX87Pascal.ScaleMatrix3f;
  ScaleMatrix3d := GeometryFunctionsX87Pascal.ScaleMatrix3d;
  ScaleMatrix4f := GeometryFunctionsX87Pascal.ScaleMatrix4f;
  ScaleMatrix4d := GeometryFunctionsX87Pascal.ScaleMatrix4d;

  MulScalarMatrix3f := GeometryFunctionsX87Pascal.MulScalarMatrix3f;
  MulScalarMatrix3d := GeometryFunctionsX87Pascal.MulScalarMatrix3d;
  MulScalarMatrix4f := GeometryFunctionsX87Pascal.MulScalarMatrix4f;
  MulScalarMatrix4d := GeometryFunctionsX87Pascal.MulScalarMatrix4d;

// *****************************************************************************
// TranslationMatrix:
// *****************************************************************************

  TranslationMatrix3f := GeometryFunctionsX87Pascal.TranslationMatrix3f;
  TranslationMatrix3d := GeometryFunctionsX87Pascal.TranslationMatrix3d;
  TranslationMatrix4f := GeometryFunctionsX87Pascal.TranslationMatrix4f;
  TranslationMatrix4d := GeometryFunctionsX87Pascal.TranslationMatrix4d;

  XYTranslationMatrix3f  := GeometryFunctionsX87Pascal.XYTranslationMatrix3f;
  XYTranslationMatrix3d  := GeometryFunctionsX87Pascal.XYTranslationMatrix3d;
  XYZTranslationMatrix4f := GeometryFunctionsX87Pascal.XYZTranslationMatrix4f;
  XYZTranslationMatrix4d := GeometryFunctionsX87Pascal.XYZTranslationMatrix4d;

// *****************************************************************************
// RotationMatrix:
// *****************************************************************************

  XRotationMatrix4f := GeometryFunctionsX87Pascal.XRotationMatrix4f;
  XRotationMatrix4d := GeometryFunctionsX87Pascal.XRotationMatrix4d;
  YRotationMatrix4f := GeometryFunctionsX87Pascal.YRotationMatrix4f;
  YRotationMatrix4d := GeometryFunctionsX87Pascal.YRotationMatrix4d;
  ZRotationMatrix4f := GeometryFunctionsX87Pascal.ZRotationMatrix4f;
  ZRotationMatrix4d := GeometryFunctionsX87Pascal.ZRotationMatrix4d;

  RotationMatrix3f := GeometryFunctionsX87Pascal.RotationMatrix3f;
  RotationMatrix3d := GeometryFunctionsX87Pascal.RotationMatrix3d;
  RotationMatrix4f := GeometryFunctionsX87Pascal.RotationMatrix4f;
  RotationMatrix4d := GeometryFunctionsX87Pascal.RotationMatrix4d;

  XYRotationMatrix3f  := GeometryFunctionsX87Pascal.XYRotationMatrix3f;
  XYRotationMatrix3d  := GeometryFunctionsX87Pascal.XYRotationMatrix3d;
  XYZRotationMatrix4f := GeometryFunctionsX87Pascal.XYZRotationMatrix4f;
  XYZRotationMatrix4d := GeometryFunctionsX87Pascal.XYZRotationMatrix4d;

// *****************************************************************************
// AdjustMatrix:
// *****************************************************************************

  AdjustAxisMatrix3f := GeometryFunctionsX87Pascal.AdjustAxisMatrix3f;
  AdjustAxisMatrix3d := GeometryFunctionsX87Pascal.AdjustAxisMatrix3d;
  AdjustAxisMatrix4f := GeometryFunctionsX87Pascal.AdjustAxisMatrix4f;
  AdjustAxisMatrix4d := GeometryFunctionsX87Pascal.AdjustAxisMatrix4d;

// *****************************************************************************
// DeterminantMatrix:
// *****************************************************************************

  DeterminantMatrix3f := GeometryFunctionsX87Pascal.DeterminantMatrix3f;
  DeterminantMatrix3d := GeometryFunctionsX87Pascal.DeterminantMatrix3d;
  DeterminantMatrix4f := GeometryFunctionsX87Pascal.DeterminantMatrix4f;
  DeterminantMatrix4d := GeometryFunctionsX87Pascal.DeterminantMatrix4d;

// *****************************************************************************
// AdjointMatrix:
// *****************************************************************************

  AdjointMatrix4f := GeometryFunctionsX87Pascal.AdjointMatrix4f;
  AdjointMatrix4d := GeometryFunctionsX87Pascal.AdjointMatrix4d;

// *****************************************************************************
// InverseMatrix:
// *****************************************************************************

  InverseMatrix3f := GeometryFunctionsX87Pascal.InverseMatrix3f;
  InverseMatrix3d := GeometryFunctionsX87Pascal.InverseMatrix3d;
  InverseMatrix4f := GeometryFunctionsX87Pascal.InverseMatrix4f;
  InverseMatrix4d := GeometryFunctionsX87Pascal.InverseMatrix4d;

//  Transformations:

// *****************************************************************************
// TransformVector:
// *****************************************************************************

  TransformVector3f := GeometryFunctionsX87Pascal.TransformVector3f;
  TransformVector3d := GeometryFunctionsX87Pascal.TransformVector3d;
  TransformVector4f := GeometryFunctionsX87Pascal.TransformVector4f;
  TransformVector4d := GeometryFunctionsX87Pascal.TransformVector4d;

  M4fTransformVector3f := GeometryFunctionsX87Pascal.M4fTransformVector3f;
  M4dTransformVector3d := GeometryFunctionsX87Pascal.M4dTransformVector3d;

  TranspTransformVector3f := GeometryFunctionsX87Pascal.TranspTransformVector3f;
  TranspTransformVector3d := GeometryFunctionsX87Pascal.TranspTransformVector3d;
  TranspTransformVector4f := GeometryFunctionsX87Pascal.TranspTransformVector4f;
  TranspTransformVector4d := GeometryFunctionsX87Pascal.TranspTransformVector4d;

  M4fTranspTransformVector3f := GeometryFunctionsX87Pascal.M4fTranspTransformVector3f;
  M4dTranspTransformVector3d := GeometryFunctionsX87Pascal.M4dTranspTransformVector3d;

  GeometryFunctionsMode := gfX87Pascal;
end;

procedure SetX87GeometryFunctionsMode;
begin NI end;

procedure Set3DNowGeometryFunctionsMode;
begin NI end;

procedure SetOptimalGeometryFunctionsMode;
begin
  SetGeometryFunctionsMode(GetOptimalGeometryFunctionsMode);
end;

function GetOptimalGeometryFunctionsMode : TGeometryFunctionsMode;
begin
  Result := gfX87Pascal; // will be changed
end;

procedure SetGeometryFunctionsMode(Mode: TGeometryFunctionsMode);
begin
  case Mode of
    gfOptimal : SetOptimalGeometryFunctionsMode;
    gfX87Pascal : SetX87PascalGeometryFunctionsMode;
    gfX87 : SetX87GeometryFunctionsMode;
    gf3DNow : Set3DNowGeometryFunctionsMode;
  end;
end;

function GetGeometryFunctionsMode : TGeometryFunctionsMode;
begin
  Result := GeometryFunctionsMode;
end;

begin
  SetOptimalGeometryFunctionsMode;
end.
