unit GeometryTypes;

interface

type
  Float = Single;

  TScalarf = Float;
  TScalard = Double;

  TAnglef = Float;
  TAngled = Double;

// *****************************************************************************
// TVector2x:
// *****************************************************************************

type
  TVector2i = array [0..1] of Integer;
  TVector2f = array [0..1] of Float;
  TVector2d = array [0..1] of Double;
  TVector2p = array [0..1] of Pointer;
const
  NullVector2i : TVector2i = (0,0);
  NullVector2f : TVector2f = (0,0);
  NullVector2d : TVector2d = (0,0);
  NullVector2p : TVector2p = (nil,nil);
  XVector2f : TVector2f = (1,0);
  XVector2d : TVector2d = (1,0);
  YVector2f : TVector2f = (0,1);
  YVector2d : TVector2d = (0,1);

function CreateVector2i(x,y: Integer) : TVector2i;
function CreateVector2f(x,y: Float) : TVector2f;
function CreateVector2d(x,y: Double) : TVector2d;
function CreateVector2p(x,y: Pointer) : TVector2p;

// *****************************************************************************
// TVector3x:
// *****************************************************************************

type
  TVector3i = array [0..2] of Integer;
  TVector3f = array [0..2] of Float;
  TVector3d = array [0..2] of Double;
  TVector3p = array [0..2] of Pointer;
const
  NullVector3i : TVector3i = (0,0,0);
  NullVector3f : TVector3f = (0,0,0);
  NullVector3d : TVector3d = (0,0,0);
  NullVector3p : TVector3p = (nil,nil,nil);
  XVector3f : TVector3f = (1,0,0);
  XVector3d : TVector3d = (1,0,0);
  YVector3f : TVector3f = (0,1,0);
  YVector3d : TVector3d = (0,1,0);
  ZVector3f : TVector3f = (0,0,1);
  ZVector3d : TVector3d = (0,0,1);
const
  NullHomVector3f : TVector3f = (0,0,1);
  NullHomVector3d : TVector3d = (0,0,1);
  XHomVector3f : TVector3f = (1,0,1);
  XHomVector3d : TVector3d = (1,0,1);
  YHomVector3f : TVector3f = (0,1,1);
  YHomVector3d : TVector3d = (0,1,1);

function CreateVector3i(x,y,z: Integer) : TVector3i;
function CreateVector3f(x,y,z: Float) : TVector3f;
function CreateVector3d(x,y,z: Double) : TVector3d;
function CreateVector3p(x,y,z: Pointer) : TVector3p;

function CreateHomVector3f(x,y: Float) : TVector3f;
function CreateHomVector3d(x,y: Double) : TVector3d;

// *****************************************************************************
// TVector4x:
// *****************************************************************************

type
  TVector4i = array [0..3] of Integer;
  TVector4f = array [0..3] of Float;
  TVector4d = array [0..3] of Double;
  TVector4p = array [0..3] of Pointer;
const
  NullVector4i : TVector4i = (0,0,0,0);
  NullVector4f : TVector4f = (0,0,0,0);
  NullVector4d : TVector4d = (0,0,0,0);
  NullVector4p : TVector4p = (nil,nil,nil,nil);
const
  NullHomVector4f : TVector4f = (0,0,0,1);
  NullHomVector4d : TVector4d = (0,0,0,1);
  XHomVector4f : TVector4f = (1,0,0,1);
  XHomVector4d : TVector4d = (1,0,0,1);
  YHomVector4f : TVector4f = (0,1,0,1);
  YHomVector4d : TVector4d = (0,1,0,1);
  ZHomVector4f : TVector4f = (0,0,1,1);
  ZHomVector4d : TVector4d = (0,0,1,1);

function CreateVector4i(x,y,z,w: Integer) : TVector4i;
function CreateVector4f(x,y,z,w: Float) : TVector4f;
function CreateVector4d(x,y,z,w: Double) : TVector4d;
function CreateVector4p(x,y,z,w: Pointer) : TVector4p;

function CreateHomVector4f(x,y,z: Float) : TVector4f;
function CreateHomVector4d(x,y,z: Double) : TVector4d;

// *****************************************************************************
// TQuaternion:  q = [w,[x,y,z]]
// *****************************************************************************

type
  TQuaternioni = packed record
    case byte of
      0: (q: TVector4i);
      1: (s: integer; v: TVector3i);
      2: (w, x, y, z: integer);
  end;

  TQuaternionf = packed record
    case byte of
      0: (q: TVector4f);
      1: (s: float; v: TVector3f);
      2: (w, x, y, z: float);
  end;

  TQuaterniond = packed record
    case byte of
      0: (q: TVector4d);
      1: (s: double; v: TVector3d);
      2: (w, x, y, z: double);
  end;

  TQuaternionp = packed record
    case byte of
      0: (q: TVector4p);
      1: (s: pointer; v: TVector3p);
      2: (w, x, y, z: pointer);
  end;
const
  NullQuaternioni : TQuaternioni = (w:0;x:0;y:0;z:0);
  NullQuaternionf : TQuaternionf = (w:0;x:0;y:0;z:0);
  NullQuaterniond : TQuaterniond = (w:0;x:0;y:0;z:0);
  NullQuaternionp : TQuaternionp = (w:nil;x:nil;y:nil;z:nil);
const
  IdentityQuaternioni : TQuaternioni = (w:1;x:0;y:0;z:0);
  IdentityQuaternionf : TQuaternionf = (w:1;x:0;y:0;z:0);
  IdentityQuaterniond : TQuaterniond = (w:1;x:0;y:0;z:0);

function CreateQuaternionf(w,x,y,z: Float): TQuaternionf;
function CreateQuaterniond(w,x,y,z: Double): TQuaterniond;

// *****************************************************************************
// TMatrix2x:
// *****************************************************************************

type
  TMatrix2i = array [0..1,0..1] of Integer;
  TMatrix2f = array [0..1,0..1] of Float;
  TMatrix2d = array [0..1,0..1] of Double;
const
  NullMatrix2i : TMatrix2i = (
    (0,0),
    (0,0)
  );
  IdentityMatrix2i : TMatrix2i = (
    (1,0),
    (0,1)
  );

  NullMatrix2f : TMatrix2f = (
    (0,0),
    (0,0)
  );
  IdentityMatrix2f : TMatrix2f = (
    (1,0),
    (0,1)
  );

  NullMatrix2d : TMatrix2d = (
    (0,0),
    (0,0)
  );
  IdentityMatrix2d : TMatrix2d = (
    (1,0),
    (0,1)
  );

function CreateMatrix2f(a1,a2,b1,b2,c1,c2: Float): TMatrix2f;
function CreateMatrix2d(a1,a2,b1,b2,c1,c2: Double): TMatrix2d;

// *****************************************************************************
// TMatrix3x:
// *****************************************************************************

type
  TMatrix3i = array [0..2,0..2] of Integer;
  TMatrix3f = array [0..2,0..2] of Float;
  TMatrix3d = array [0..2,0..2] of Double;
const
  NullMatrix3i : TMatrix3i = (
    (0,0,0),
    (0,0,0),
    (0,0,0)
  );
  IdentityMatrix3i : TMatrix3i = (
    (1,0,0),
    (0,1,0),
    (0,0,1)
  );

  NullMatrix3f : TMatrix3f = (
    (0,0,0),
    (0,0,0),
    (0,0,0)
  );
  IdentityMatrix3f : TMatrix3f = (
    (1,0,0),
    (0,1,0),
    (0,0,1)
  );

  NullMatrix3d : TMatrix3d = (
    (0,0,0),
    (0,0,0),
    (0,0,0)
  );
  IdentityMatrix3d : TMatrix3d = (
    (1,0,0),
    (0,1,0),
    (0,0,1)
  );

function CreateMatrix3f(a1,a2,a3,b1,b2,b3,c1,c2,c3: Float): TMatrix3f;
function CreateMatrix3d(a1,a2,a3,b1,b2,b3,c1,c2,c3: Double): TMatrix3d;

// *****************************************************************************
// TMatrix4x:
// *****************************************************************************

type
  TMatrix4i = array [0..3,0..3] of Integer;
  TMatrix4f = array [0..3,0..3] of Float;
  TMatrix4d = array [0..3,0..3] of Double;
const
  NullMatrix4i : TMatrix4i = (
    (0,0,0,0),
    (0,0,0,0),
    (0,0,0,0),
    (0,0,0,0)
  );
  IdentityMatrix4i : TMatrix4i = (
    (1,0,0,0),
    (0,1,0,0),
    (0,0,1,0),
    (0,0,0,1)
  );

  NullMatrix4f : TMatrix4f = (
    (0,0,0,0),
    (0,0,0,0),
    (0,0,0,0),
    (0,0,0,0)
  );
  IdentityMatrix4f : TMatrix4f = (
    (1,0,0,0),
    (0,1,0,0),
    (0,0,1,0),
    (0,0,0,1)
  );

  NullMatrix4d : TMatrix4d = (
    (0,0,0,0),
    (0,0,0,0),
    (0,0,0,0),
    (0,0,0,0)
  );
  IdentityMatrix4d : TMatrix4d = (
    (1,0,0,0),
    (0,1,0,0),
    (0,0,1,0),
    (0,0,0,1)
  );

function CreateMatrix4f(a1,a2,a3,a4,b1,b2,b3,b4,c1,c2,c3,c4,d1,d2,d3,d4: Float): TMatrix4f;
function CreateMatrix4d(a1,a2,a3,a4,b1,b2,b3,b4,c1,c2,c3,c4,d1,d2,d3,d4: Double): TMatrix4d;

// *****************************************************************************
// TPointXy:
// *****************************************************************************

type
  TPoint2i = TVector2i;
  TPoint2f = TVector2f;
  TPoint2d = TVector2d;
  TPoint3i = TVector3i;
  TPoint3f = TVector3f;
  TPoint3d = TVector3d;
  TPoint4i = TVector4i;
  TPoint4f = TVector4f;
  TPoint4d = TVector4d;

// *****************************************************************************
// TColorXy:
// *****************************************************************************

type
  TColor3b = array [0..2] of Byte;
  TColor3w = array [0..2] of Word;
  TColor3f = array [0..2] of Float;
  TColor3d = array [0..2] of Double;
  TColor3i = array [0..2] of integer;

const
  BlackColor3b : TColor3b = (0,0,0);
  BlackColor3w : TColor3w = (0,0,0);
  BlackColor3f : TColor3f = (0,0,0);
  WhiteColor3b : TColor3b = ($ff,$ff,$ff);
  WhiteColor3w : TColor3w = ($ffff,$ffff,$ffff);
  WhiteColor3f : TColor3f = (1,1,1);

function CreateColor3b(r,g,b: Byte) : TColor3b;
function CreateColor3w(r,g,b: Word) : TColor3w;
function CreateColor3f(r,g,b: Float) : TColor3f;

type
  TColor4b = array [0..3] of Byte;
  TColor4w = array [0..3] of Word;
  TColor4f = array [0..3] of Float;
  TColor4d = array [0..3] of Double;
  TColor4i = array [0..3] of integer;


const
  BlackColor4b : TColor4b = (0,0,0,$ff);
  BlackColor4w : TColor4w = (0,0,0,$ffff);
  BlackColor4f : TColor4f = (0,0,0,1);
  TransparentColor4b : TColor4b = (0,0,0,0);
  TransparentColor4w : TColor4w = (0,0,0,0);
  TransparentColor4f : TColor4f = (0,0,0,0);
  WhiteColor4b : TColor4b = ($ff,$ff,$ff,$ff);
  WhiteColor4w : TColor4w = ($ffff,$ffff,$ffff,$ffff);
  WhiteColor4f : TColor4f = (1,1,1,1);
  // The default OpenGL lighting values:
  DefaultAmbientColor4b : TColor4b = (51,51,51,255);
  DefaultAmbientColor4w : TColor4w = (13107,13107,13107,65535);
  DefaultAmbientColor4f : TColor4f = (0.2,0.2,0.2,1.0);
  DefaultDiffuseColor4b : TColor4b = (204,204,204,255);
  DefaultDiffuseColor4w : TColor4w = (52428,52428,52428,65535);
  DefaultDiffuseColor4f : TColor4f = (0.8,0.8,0.8,1.0);

function CreateColor4b(r,g,b,a: Byte) : TColor4b;
function CreateColor4w(r,g,b,a: Word) : TColor4w;
function CreateColor4f(r,g,b,a: Float) : TColor4f;

implementation

function CreateVector2i(x,y: Integer) : TVector2i;
begin
  Result[0] := x;
  Result[1] := y;
end;

function CreateVector2f(x,y: Float) : TVector2f;
begin
  Result[0] := x;
  Result[1] := y;
end;

function CreateVector2d(x,y: Double) : TVector2d;
begin
  Result[0] := x;
  Result[1] := y;
end;

function CreateVector2p(x,y: Pointer) : TVector2p;
begin
  Result[0] := x;
  Result[1] := y;
end;


function CreateVector3i(x,y,z: Integer) : TVector3i;
begin
  Result[0] := x;
  Result[1] := y;
  Result[2] := z;
end;

function CreateVector3f(x,y,z: Float) : TVector3f;
begin
  Result[0] := x;
  Result[1] := y;
  Result[2] := z;
end;

function CreateVector3d(x,y,z: Double) : TVector3d;
begin
  Result[0] := x;
  Result[1] := y;
  Result[2] := z;
end;

function CreateVector3p(x,y,z: Pointer) : TVector3p;
begin
  Result[0] := x;
  Result[1] := y;
  Result[2] := z;
end;

function CreateHomVector3f(x,y: Float) : TVector3f;
begin
  Result[0] := x;
  Result[1] := y;
  Result[2] := 1;
end;

function CreateHomVector3d(x,y: Double) : TVector3d;
begin
  Result[0] := x;
  Result[1] := y;
  Result[2] := 1;
end;

function CreateVector4i(x,y,z,w: Integer) : TVector4i;
begin
  Result[0] := x;
  Result[1] := y;
  Result[2] := z;
  Result[3] := w;
end;

function CreateVector4f(x,y,z,w: Float) : TVector4f;
begin
  Result[0] := x;
  Result[1] := y;
  Result[2] := z;
  Result[3] := w;
end;

function CreateVector4d(x,y,z,w: Double) : TVector4d;
begin
  Result[0] := x;
  Result[1] := y;
  Result[2] := z;
  Result[3] := w;
end;

function CreateVector4p(x,y,z,w: Pointer) : TVector4p;
begin
  Result[0] := x;
  Result[1] := y;
  Result[2] := z;
  Result[3] := w;
end;

function CreateHomVector4f(x,y,z: Float) : TVector4f;
begin
  Result[0] := x;
  Result[1] := y;
  Result[2] := z;
  Result[3] := 1;
end;

function CreateHomVector4d(x,y,z: Double) : TVector4d;
begin
  Result[0] := x;
  Result[1] := y;
  Result[2] := z;
  Result[3] := 1;
end;

function CreateColor3b(r,g,b: Byte) : TColor3b;
begin
  Result[0] := r;
  Result[1] := g;
  Result[2] := b;
end;

function CreateColor3w(r,g,b: Word) : TColor3w;
begin
  Result[0] := r;
  Result[1] := g;
  Result[2] := b;
end;

function CreateColor3f(r,g,b: Float) : TColor3f;
begin
  Result[0] := r;
  Result[1] := g;
  Result[2] := b;
end;

function CreateColor4b(r,g,b,a: Byte) : TColor4b;
begin
  Result[0] := r;
  Result[1] := g;
  Result[2] := b;
  Result[3] := a;
end;

function CreateColor4w(r,g,b,a: Word) : TColor4w;
begin
  Result[0] := r;
  Result[1] := g;
  Result[2] := b;
  Result[3] := a;
end;

function CreateColor4f(r,g,b,a: Float) : TColor4f;
begin
  Result[0] := r;
  Result[1] := g;
  Result[2] := b;
  Result[3] := a;
end;

function CreateMatrix2f(a1,a2,b1,b2,c1,c2: Float): TMatrix2f;
begin
  Result[0,0] := a1; Result[0,1] := a2;
  Result[1,0] := b1; Result[1,1] := b2;
end;

function CreateMatrix2d(a1,a2,b1,b2,c1,c2: Double): TMatrix2d;
begin
  Result[0,0] := a1; Result[0,1] := a2;
  Result[1,0] := b1; Result[1,1] := b2;
end;

function CreateMatrix3f(a1,a2,a3,b1,b2,b3,c1,c2,c3: Float): TMatrix3f;
begin
  Result[0,0] := a1; Result[0,1] := a2; Result[0,2] := a3;
  Result[1,0] := b1; Result[1,1] := b2; Result[1,2] := b3;
  Result[2,0] := c1; Result[2,1] := c2; Result[2,2] := c3;
end;

function CreateMatrix3d(a1,a2,a3,b1,b2,b3,c1,c2,c3: Double): TMatrix3d;
begin
  Result[0,0] := a1; Result[0,1] := a2; Result[0,2] := a3;
  Result[1,0] := b1; Result[1,1] := b2; Result[1,2] := b3;
  Result[2,0] := c1; Result[2,1] := c2; Result[2,2] := c3;
end;

function CreateMatrix4f(a1,a2,a3,a4,b1,b2,b3,b4,c1,c2,c3,c4,d1,d2,d3,d4: Float): TMatrix4f;
begin
  Result[0,0] := a1; Result[0,1] := a2; Result[0,2] := a3; Result[0,3] := a4;
  Result[1,0] := b1; Result[1,1] := b2; Result[1,2] := b3; Result[1,3] := b4;
  Result[2,0] := c1; Result[2,1] := c2; Result[2,2] := c3; Result[2,3] := c4;
  Result[3,0] := d1; Result[3,1] := d2; Result[3,2] := d3; Result[3,3] := d4;
end;

function CreateMatrix4d(a1,a2,a3,a4,b1,b2,b3,b4,c1,c2,c3,c4,d1,d2,d3,d4: Double): TMatrix4d;
begin
  Result[0,0] := a1; Result[0,1] := a2; Result[0,2] := a3; Result[0,3] := a4;
  Result[1,0] := b1; Result[1,1] := b2; Result[1,2] := b3; Result[1,3] := b4;
  Result[2,0] := c1; Result[2,1] := c2; Result[2,2] := c3; Result[2,3] := c4;
  Result[3,0] := d1; Result[3,1] := d2; Result[3,2] := d3; Result[3,3] := d4;
end;

function CreateQuaternionf(w,x,y,z: Float): TQuaternionf;
begin
  Result.q[0] := w;
  Result.q[1] := x;
  Result.q[2] := y;
  Result.q[3] := z;
end;

function CreateQuaterniond(w,x,y,z: Double): TQuaterniond;
begin
  Result.q[0] := w;
  Result.q[1] := x;
  Result.q[2] := y;
  Result.q[3] := z;
end;



end.
