{*******************************************************}
{                                                       }
{                  TIndexList class                     }
{                                                       }
{           Copyright (c) 2000 by Serge Gusev           }
{                                                       }
{*******************************************************}

unit IndexLst;

interface

uses Classes, Dialogs;

type
  { TIndexItem }

  TIndexList = class;

  TIndexItem = class(TObject)
  private
    FList: TIndexList;
    FKeys: TList;
    FData: TList;
    FParent: TIndexItem;
    FLeft: TIndexItem;
    FRight: TIndexItem;
    FCount: Integer;
    FLeftCount: Integer;
    FRightCount: Integer;
    function GetKey(Index: Integer): Integer;
    function GetData(Index: Integer): Pointer;
    function Add(Key: Integer; out RealAdd, ItemAdd: Boolean): Pointer;
    function Remove(Key: Integer; out ItemRemove: Boolean): TIndexItem;
    function Delete(Index: Integer; out ItemRemove: Boolean): TIndexItem;
    procedure CheckBalance;
    function MostItem: TIndexItem;
    function LeastItem: TIndexItem;
    procedure AddItem(Item: TIndexItem);
    function RemoveItem(Item: TIndexItem): TIndexItem;
    function IndexOf(Key: Integer): Integer;
    procedure Clear;
    function GetLeftCount: Longint;
    function GetRightCount: Longint;
  protected
    function IsTop: Boolean;
    function IsLeft: Boolean;
    function IsRight: Boolean;
  public
    constructor Create(AList: TIndexList);
    destructor Destroy; override;
    property Count: Integer read FCount;
    property Keys[Index: Integer]: Integer read GetKey;
    property Data[Index: Integer]: Pointer read GetData; default;
    property Parent: TIndexItem read FParent;
    property Left: TIndexItem read FLeft;
    property Right: TIndexItem read FRight;
    property LeftCount: Integer read FLeftCount;
    property RightCount: Integer read FRightCount;
  end;

  { TIndexList }

  TIndexList = class(TObject)
  private
    FItems: TIndexItem;
    function GetCount: Integer;
    function GetKey(Index: Integer): Integer;
    function GetData(Index: Integer): Pointer;
  protected
    function CreateData: Pointer; virtual;
    procedure FreeData(AData: Pointer); virtual;
    procedure AssignData(Source, Dest: Pointer); virtual;
  public
    constructor Create;
    destructor Destroy; override;
    function Add(Key: Integer): Pointer;
    procedure Remove(Key: Integer);
    procedure Delete(Index: Integer);
    function IndexOf(Key: Integer): Integer;
    procedure Clear;
    property Count: Integer read GetCount;
    property Keys[Index: Integer]: Integer read GetKey;
    property Data[Index: Integer]: Pointer read GetData; default;
    property Items: TIndexItem read FItems;
  end;

implementation

const
  BufferSize = 256;

{ TIndexItem }

constructor TIndexItem.Create(AList: TIndexList);
begin
  inherited Create;
  FList := AList;
  FKeys := TList.Create;
  FKeys.Capacity := BufferSize;
  FillChar(FKeys.List^, BufferSize, 0);
  FData := TList.Create;
  FData.Capacity := BufferSize;
  FillChar(FKeys.List^, BufferSize, 0);
  FParent := nil;
  FLeft := nil;
  FRight := nil;
  FLeftCount := 0;
  FRightCount := 0
end;

destructor TIndexItem.Destroy;
var
  I: Integer;
begin
  FLeft.Free;
  FRight.Free;
  if FParent <> nil then RemoveItem(Self);
  if FList <> nil then
    for I := 0 to FKeys.Count - 1 do FList.FreeData(FData[I]);
  FData.Free;
  FKeys.Free;  
  inherited Destroy
end;

function TIndexItem.GetKey(Index: Integer): Integer;
var
  I: Integer;
begin
  I := Index;
  if FLeft <> nil then Dec(I, FLeft.Count);
  if I < 0 then Result := FLeft.Keys[Index]
  else
    if I < FKeys.Count then Result := Longint(FKeys[I])
    else
      if FRight <> nil then
      begin
        Dec(I, FKeys.Count);
        Result := FRight.Keys[I]
      end
      else Result := 0
end;

function TIndexItem.GetData(Index: Integer): Pointer;
var
  I: Integer;
begin
  I := Index;
  if FLeft <> nil then Dec(I, FLeft.Count);
  if I < 0 then Result := FLeft.Data[Index]
  else
    if I < FData.Count then Result := FData[I]
    else
      if FRight <> nil then
      begin
        Dec(I, FData.Count);
        Result := FRight.Data[I]
      end
      else Result := nil
end;

function TIndexItem.Add(Key: Integer; out RealAdd, ItemAdd: Boolean): Pointer;
var
  I: Integer;
  B, C: Boolean;
begin
  ItemAdd := False;
  RealAdd := True;
  Result := nil;
  B := False;
  if FKeys.Count = 0 then
  begin
    FKeys.Add(Pointer(Key));
    FData.Add(FList.CreateData);
    Result := FData[0]
  end
  else
    if Key > Integer(FKeys[FKeys.Count - 1]) then
      if FKeys.Count >= BufferSize then
      begin
        if FRight = nil then
        begin
          FRight := TIndexItem.Create(FList);
          FRight.FParent := Self;
          B := True
        end;
        Result := FRight.Add(Key, RealAdd, ItemAdd);
        ItemAdd := ItemAdd or B;
        if ItemAdd then Inc(FRightCount)
      end
      else
        if FRight <> nil then
        begin
          Result := FRight.Add(Key, RealAdd, ItemAdd);
          if ItemAdd then Inc(FRightCount)
        end
        else
        begin
          FKeys.Add(Pointer(Key));
          FData.Add(FList.CreateData);
          Result := FData[FKeys.Count - 1]
        end
    else
      if Key < Integer(FKeys[0]) then
        if FKeys.Count >= BufferSize then
        begin
          if FLeft = nil then
          begin
            FLeft := TIndexItem.Create(FList);
            FLeft.FParent := Self;
            B := True
          end;
          Result := FLeft.Add(Key, RealAdd, ItemAdd);
          ItemAdd := ItemAdd or B;
          if ItemAdd then Inc(FLeftCount)
        end
        else
          if FLeft <> nil then
          begin
            Result := FLeft.Add(Key, RealAdd, ItemAdd);
            if ItemAdd then Inc(FLeftCount)
          end
          else
          begin
            FKeys.Insert(0, Pointer(Key));
            FData.Insert(0, FList.CreateData);
            Result := FData[0]
          end
      else
      begin
        for I := 0 to FKeys.Count - 1 do
          if Key <= Integer(FKeys[I]) then
          begin
            if Key = Integer(FKeys[I]) then RealAdd := False
            else
            begin
              FKeys.Insert(I, Pointer(Key));
              FData.Insert(I, FList.CreateData)
            end;
            Result := FData[I];
            Break
          end;
        if FKeys.Count > BufferSize then
        begin
          if FRight = nil then
          begin
            FRight := TIndexItem.Create(FList);
            FRight.FParent := Self;
            B := True
          end;
          FList.AssignData(FData[FKeys.Count - 1],
            FRight.Add(Integer(FKeys[FKeys.Count - 1]), C, ItemAdd));
          ItemAdd := ItemAdd or B;
          if ItemAdd then Inc(FRightCount);
          Inc(FCount);
          Remove(Integer(FKeys[FKeys.Count - 1]), B)
        end
      end;
  if RealAdd then Inc(FCount);
  if ItemAdd then CheckBalance
end;

function TIndexItem.Remove(Key: Integer; out ItemRemove: Boolean): TIndexItem;
var
  I: Integer;
begin
  I := IndexOf(Key);
  if I = -1 then Result := nil
  else
    Result := Delete(I, ItemRemove)
end;

function TIndexItem.Delete(Index: Integer; out ItemRemove: Boolean): TIndexItem;
var
  I: Integer;
begin
  Result := nil;
  ItemRemove := False;
  I := Index;
  if FLeft <> nil then Dec(I, FLeft.Count);
  if I < 0 then
  begin
    Result := FLeft.Delete(Index, ItemRemove);
    if ItemRemove then Dec(FLeftCount)
  end
  else
    if I < FKeys.Count then
    begin
      if FKeys.Count = 1 then
      begin
        Result := Self;
        ItemRemove := True;
        RemoveItem(Self)
      end;
      FKeys.Delete(I);
      FList.FreeData(FData[I]);
      FData.Delete(I)
    end
    else
      if FRight <> nil then
      begin
        Dec(I, FData.Count);
        Result := FRight.Delete(I, ItemRemove);
        if ItemRemove then Dec(FRightCount)
      end
      else Result := nil;
  Dec(FCount);    
  if ItemRemove then CheckBalance    
end;

function TIndexItem.GetLeftCount: Integer;
begin
  if FLeft = nil then Result := 0
  else
    Result := 1 + FLeft.GetRightCount + FLeft.GetLeftCount
end;

function TIndexItem.GetRightCount: Integer;
begin
  if FRight = nil then Result := 0
  else
    Result := 1 + FRight.GetRightCount + FRight.GetLeftCount
end;

procedure TIndexItem.CheckBalance;
var
  Item: TIndexItem;
begin
  while Abs(FLeftCount - FRightCount) > 1 do
  begin
    Item := RemoveItem(Self);
    if Item <> nil then Item.AddItem(Self)
  end
end;

function TIndexItem.MostItem: TIndexItem;
begin
  if FRight = nil then Result := Self
  else
    Result := FRight.MostItem
end;

function TIndexItem.LeastItem: TIndexItem;
begin
  if FLeft = nil then Result := Self
  else
    Result := FLeft.LeastItem
end;

procedure TIndexItem.AddItem(Item: TIndexItem);
begin
  if (Item = nil) or (FList = nil) then Exit;
  if Integer(Item.FKeys[0]) < Integer(FKeys[0]) then
  begin
    if FLeft <> nil then FLeft.AddItem(Item)
    else
    begin
      FLeft := Item;
      Item.FList := FList;
      Item.FParent := Self
    end;
    Inc(FLeftCount)
  end
  else
  begin
    if FRight <> nil then FRight.AddItem(Item)
    else
    begin
      FRight := Item;
      Item.FList := FList;
      Item.FParent := Self
    end;
    Inc(FRightCount)
  end;
  Inc(FCount, Item.FCount);
  CheckBalance
end;

function TIndexItem.RemoveItem(Item: TIndexItem): TIndexItem;

  function GetReplaceItem(Item: TIndexItem): TIndexItem;
  begin
    if (Item.FLeftCount > Item.FRightCount) and (FLeft <> nil) then
      Result := Item.FLeft.MostItem
    else
      if Item.FRight = nil then Result := nil
      else
        Result := Item.FRight.LeastItem;
    if Result <> nil then
    begin
      Item.RemoveItem(Result);
      Result.FRight := Item.FRight;
      if Item.FRight <> nil then
      begin
        Item.FRight.FParent := Result;
        Inc(Result.FCount, Item.FRight.FCount)
      end;
      Result.FLeft := Item.FLeft;
      if Item.FLeft <> nil then
      begin
        Item.FLeft.FParent := Result;
        Inc(Result.FCount, Item.FLeft.FCount)
      end;
      Result.FLeftCount := Item.FLeftCount;
      Result.FRightCount := Item.FRightCount;
      Result.FParent := Item.FParent
    end
  end;

begin
  Result := nil;
  if FList = nil then Exit;
  if Integer(Item.FKeys[0]) < Integer(FKeys[0]) then
  begin
    if FLeft = Item then
    begin
      Result := GetReplaceItem(Item);
      FLeft := Result
    end
    else
      if FLeft <> nil then
        Result := FLeft.RemoveItem(Item);
    Dec(FLeftCount)
  end
  else
    if Integer(Item.FKeys[0]) > Integer(FKeys[0]) then
    begin
      if FRight = Item then
      begin
        Result := GetReplaceItem(Item);
        FRight := Result;
      end
      else
        if FRight <> nil then
          Result := FRight.RemoveItem(Item);
      Dec(FRightCount)
    end
    else
    begin
      Result := GetReplaceItem(Item);
      if FParent <> nil then
        if IsLeft then FParent.FLeft := Result
        else
          FParent.FRight := Result;
      if IsTop then FList.FItems := Result
    end;
  Item.Clear;
  if Item <> Self then
  begin
    Dec(FCount, Item.FCount);
    CheckBalance
  end
end;

function TIndexItem.IndexOf(Key: Integer): Integer;
begin
  Result := -1;
  if Key < Integer(FKeys[0]) then
  begin
    if FLeft <> nil then Result := FLeft.IndexOf(Key)
  end
  else
    if Key > Integer(FKeys[FKeys.Count - 1]) then
    begin
      if FRight <> nil then
      begin
        Result := FRight.IndexOf(Key);
        if Result <> -1 then
        begin
          Inc(Result, FKeys.Count);
          if FLeft <> nil then Inc(Result, FLeft.Count)
        end
      end
    end
    else
    begin
      Result := FKeys.IndexOf(Pointer(Key));
      if (Result <> -1) and (FLeft <> nil) then
        Inc(Result, FLeft.Count)
    end
end;

procedure TIndexItem.Clear;
begin
  FParent := nil;
  FLeft := nil;
  FRight := nil;
  FCount := FKeys.Count;
  FLeftCount := 0;
  FRightCount := 0
end;

function TIndexItem.IsTop: Boolean;
begin
  Result := (FList <> nil) and (FList.FItems = Self)
end;

function TIndexItem.IsLeft: Boolean;
begin
  Result := (FParent <> nil) and (FParent.FLeft = Self)
end;

function TIndexItem.IsRight: Boolean;
begin
  Result := (FParent <> nil) and (FParent.FRight = Self)
end;

{ TIndexList }

constructor TIndexList.Create;
begin
  inherited Create;
  FItems := nil
end;

destructor TIndexList.Destroy;
begin
  FItems.Free;
  inherited Destroy
end;

function TIndexList.CreateData: Pointer;
begin
  Result := nil
end;

procedure TIndexList.FreeData(AData: Pointer);
begin
end;

procedure TIndexList.AssignData(Source, Dest: Pointer);
begin
end;

function TIndexList.GetCount: Integer;
begin
  if FItems = nil then Result := 0
  else
    Result := FItems.Count
end;

function TIndexList.GetKey(Index: Integer): Integer;
begin
  if FItems = nil then Result := 0
  else
    Result := FItems.Keys[Index]
end;

function TIndexList.GetData(Index: Integer): Pointer;
begin
  if FItems = nil then Result := nil
  else
    Result := FItems[Index]
end;

function TIndexList.Add(Key: Integer): Pointer;
var
  B, C: Boolean;
begin
  if FItems = nil then FItems := TIndexItem.Create(Self);
  Result := FItems.Add(Key, B, C)
end;

procedure TIndexList.Remove(Key: Integer);
var
  B: Boolean;
begin
  if FItems <> nil then FItems.Remove(Key, B).Free
end;

procedure TIndexList.Delete(Index: Integer);
var
  B: Boolean;
begin
  if FItems <> nil then FItems.Delete(Index, B).Free
end;

function TIndexList.IndexOf(Key: Integer): Integer;
begin
  if FItems = nil then Result := -1
  else
    Result := FItems.IndexOf(Key)
end;

procedure TIndexList.Clear;
begin
  FItems.Free;
  FItems := nil
end;

end.
