{*******************************************************}
{                                                       }
{              Memory inspector project                 }
{                                                       }
{                  TMemoryStateForm                     }
{                                                       }
{           Copyright (c) 1999 Serge Gusev              }
{                                                       }
{*******************************************************}

unit MemState;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Grids, ExtCtrls, FreeWin;

procedure ShowMemoryState;

implementation

uses MemInsp;

type

  { TMemoryStateForm }

  TMemoryStateForm = class(TFreeForm)
  private
    FMemObjectsBox: TFreeListBox;
    FCallStackBox: TFreeComboBox;
    procedure FormClose(Sender: TObject);
    procedure MemObjectsBoxClick(Sender: TObject);
    procedure WMMeasureItem(var Message: TWMMeasureItem); message WM_MEASUREITEM;
    procedure WMDrawItem(var Message: TWMDrawItem); message WM_DRAWITEM;
  public
    constructor Create(AParent: TFreeWinControl; AParentWindow: HWND); override;
  end;

var
  IsAboutMode: Boolean = True;

constructor TMemoryStateForm.Create(AParent: TFreeWinControl; AParentWindow: HWND);
var
  FntData: TFontData;
  CallStackBox: TFreeGroupBox;
  MemItems: TStringList;
  S: string;
  I: Integer;
begin
  inherited Create(AParent, AParentWindow);
  if IsAboutMode then
  begin
    Caption := 'Warning!';
    Height := 130;
    with TFreeStatic.Create(Self, 0) do
    begin
      Top := 20;
      Left := 10;
      Width := Self.Width - Left;
      Height := 34;
      FntData := FontData;
      FntData.Style := [fsBold];
      FntData.Height := -16;
      FontData := FntData;
      Caption := 'Your project is running under Memory Inspector';
    end;
    with TFreeButton.Create(Self, 0) do
    begin
      Left := (Self.Width - Width) div 2;
      Top := 60;
      Caption := 'Close';
      OnClick := FormClose;
      SetFocus(Handle)
    end
  end
  else
  begin
    Height := 406;
    Caption := 'Memory State';
    CallStackBox := TFreeGroupBox.Create(Self, 0);
    with CallStackBox do
    begin
      Width := Self.Width - 90;
      Height := 50;
      Caption := ' Call stack '
    end;
    with TFreeStatic.Create(CallStackBox, 0) do
    begin
      Left := 6;
      Top := 22;
      Width := CallStackBox.Width - 80;
      Caption := 'Address for Search/Find Error: '
    end;
    FCallStackBox := TFreeComboBox.Create(CallStackBox, 0);
    with FCallStackBox do
    begin
      Width := 110;
      Left := 195;
      Top := 20
    end;
    with TFreeButton.Create(Self, 0) do
    begin
      Left := Self.Width - 85;
      Top := 8;
      Height := 42;
      Caption := 'Close';
      OnClick := FormClose
    end;
    with TFreeGroupBox.Create(Self, 0)do
    begin
      Top := 54;
      Width := Self.Width - 6;
      Height := Self.Height - Top - GetSystemMetrics(SM_CYCAPTION) - 6;
      Caption := ' Memory objects '
    end;
    with TFreeStatic.Create(Self, 0) do
    begin
      Top := 86;
      Left := 12;
      Caption := 'Address'
    end;
    with TFreeStatic.Create(Self, 0) do
    begin
      Top := 86;
      Left := 106;
      Caption := 'Size'
    end;
    with TFreeStatic.Create(Self, 0) do
    begin
      Top := 86;
      Left := 150;
      Caption := 'Class name'
    end;
    FMemObjectsBox := TFreeListBox.Create(Self, 0);
    with FMemObjectsBox do
    begin
      Top := 106;
      Left := 6;
      Width := Self.Width - 20;
      Height := 16 * 17 + 4;
      Style := lbOwnerDrawFixed;
      OnClick := MemObjectsBoxClick;
      MemItems := TStringList.Create;
      try
        for I := 0 to GetMemObjectCount - 1 do
          with GetMemObjectInfo(I) do
          begin
            if MemObjectClass  = nil then
              S := 'Not an object'
            else
              S := MemObjectClass.ClassName;
            MemItems.Add('$' + IntToHex(Integer(Address), 8) + '|' +
              IntToStr(MemSize) + '|' + S)
          end;
        Items := MemItems;
        ItemIndex := 0;
        SetFocus(Handle)
      finally
        MemItems.Free
      end
    end
  end
end;

procedure TMemoryStateForm.FormClose(Sender: TObject);
begin
  Close
end;

procedure TMemoryStateForm.MemObjectsBoxClick(Sender: TObject);
var
  StackItems: TStringList;
  I: Integer;
begin
  if FCallStackBox = nil then Exit;
  StackItems := TStringList.Create;
  try
    with GetMemObjectInfo(FMemObjectsBox.ItemIndex) do
    begin
      for I := 0 to CallStack.Count - 1 do
        StackItems.Add('$' + IntToHex(Integer(CallStack[I]), 8));
      FCallStackBox.Items := StackItems;
      FCallStackBox.ItemIndex := 0
    end
  finally
    StackItems.Free
  end
end;

procedure TMemoryStateForm.WMMeasureItem(var Message: TWMMeasureItem);
begin
  inherited;
  with Message.MeasureItemStruct^ do
    if CtlType = ODT_LISTBOX then itemHeight := 20
end;

procedure TMemoryStateForm.WMDrawItem(var Message: TWMDrawItem);
var
  S, SubS: string;
  Brush, OldBrush: HBRUSH;
  State: TOwnerDrawState;
begin
  inherited;
  with Message.DrawItemStruct^ do
    if CtlType = ODT_LISTBOX then
    begin
      {$IFDEF VER130}
        State := TOwnerDrawState(LongRec(itemState).Lo);
      {$ELSE}
        State := TOwnerDrawState(WordRec(LongRec(itemState).Lo).Lo);
      {$ENDIF}
      if odSelected in State then
      begin
        Brush := GetSysColorBrush(COLOR_HIGHLIGHT);
        SetBkColor(hDC, GetSysColor(COLOR_HIGHLIGHT));
        SetTextColor(hDC, GetSysColor(COLOR_HIGHLIGHTTEXT))
      end
      else
      begin
        Brush := GetSysColorBrush(COLOR_WINDOW);
        SetBkColor(hDC, GetSysColor(COLOR_WINDOW));
        SetTextColor(hDC, GetSysColor(COLOR_WINDOWTEXT))
      end;
      OldBrush := SelectObject(hDC, Brush);
      try
        FillRect(hDC, rcItem, Brush);
        S := FMemObjectsBox.Items[ItemId];
        SubS := Copy(S, 0, Pos('|', S) - 1);
        TextOut(hDC, rcItem.Left + 2, rcItem.Top + 2,
          PChar(SubS), Length(SubS));
        Delete(S, 1, Pos('|', S));
        SubS := Copy(S, 0, Pos('|', S) - 1);
        TextOut(hDC, rcItem.Left + 100, rcItem.Top + 2,
          PChar(SubS), Length(SubS));
        Delete(S, 1, Pos('|', S));
        TextOut(hDC, rcItem.Left + 140, rcItem.Top + 2,
          PChar(S), Length(S));
        if odFocused in State then
          DrawFocusRect(hDC, rcItem)
      finally
        SelectObject(hDC, OldBrush);
        DeleteObject(Brush)
      end
    end
end;

procedure ShowMemoryState;
var
  FormParent: HWND;
begin
  LockManager;
  try
    if IsAboutMode or (Application = nil) then
      FormParent := GetDesktopWindow
    else
      FormParent := Application.Handle;
    with TMemoryStateForm.Create(nil, FormParent) do
    try
      ShowModal
    finally
      Free
    end;
  finally
    UnlockManager
  end
end;

procedure ShowLastMemoryState;
begin
  if GetMemObjectCount > 0 then
  begin
    LockManager;
    try
      ShowMemoryState
    finally
      UnlockManager
    end
  end
end;

var
  OldFreeManagerProc: TFreeManagerProc = nil;

procedure FreeManagerProcedure;
begin
  if Assigned(OldFreeManagerProc) then OldFreeManagerProc;
  ShowLastMemoryState
end;

initialization

  ShowMemoryState;
  IsAboutMode := False;
  OldFreeManagerProc := FreeManagerProc;
  FreeManagerProc := FreeManagerProcedure

end.
