(*
    +---------------GeometryTypes------+---------------------------------------------------------+
    |			|              V                                       		         |
    |			|              +--GeometryFunctionsX87Pascal.pas			 |
    |	       		|              |  					    	         |
    V			V	       +--GeometryFunctionsX87.pas<---GeomtryFunctionsX87.asm<---+
Geometry.pas<---GeomatryFunctions.pas<-+                                        		 |
		        	       +--GeometryFunctions3DNow.pas<-GeomtryFunctions3DNow.asm<-+
			   	       |                                         		 |
	        	 	       +--GeometryFunktionsKNI.pas<---GeomtryFunctionsKNI.asm<---+
*)

unit Geometry;

interface

uses
  GeometryTypes,GeometryFunctions;

type
  TFloatVector = array of Float;
  TDoubleVector = array of Double;

function CreateFloatVector(Elements: array of Float) : TFloatVector; // overload; not possible because of ambiguous call
function CreateDoubleVector(Elements: array of Double) : TDoubleVector;

type
  TFloatMatrix = array of array of Float;
  TDoubleMatrix = array of array of Double;

function CreateFloatMatrix(Elements: array of TFloatVector) : TFloatMatrix;
function CreateDoubleMatrix(Elements: array of TDoubleVector) : TDoubleMatrix;

type
  TByteColor = array of Byte;
  TWordColor = array of Word;
  TFloatColor = array of Float;

function CreateByteColor(Elements: array of Byte) : TByteColor;
function CreateWordColor(Elements: array of Word) : TWordColor;
function CreateFloatColor(Elements: array of Float) : TFloatColor;


function ConvertVector(Vector: TVector2f) : TFloatVector; overload
function ConvertVector(Vector: TVector3f) : TFloatVector; overload
function ConvertVector(Vector: TVector4f) : TFloatVector; overload
function ConvertVector(Vector: TVector2d) : TDoubleVector; overload
function ConvertVector(Vector: TVector3d) : TDoubleVector; overload
function ConvertVector(Vector: TVector4d) : TDoubleVector; overload




// only proof of concept...

function AddVector(const v1,v2: TFloatVector) : TFloatVector; overload;
function AddVector(const v1,v2: TDoubleVector) : TDoubleVector; overload;










implementation

function CreateFloatVector(Elements: array of Float) : TFloatVector;
var
  i : Integer;
begin
  SetLength(Result, Length(Elements));
  for i := Low(Elements) to High(Elements) do
    Result[i] := Elements[i];
end;

function CreateDoubleVector(Elements: array of Double) : TDoubleVector;
var
  i : Integer;
begin
  SetLength(Result, Length(Elements));
  for i := Low(Elements) to High(Elements) do
    Result[i] := Elements[i];
end;

function CreateFloatMatrix(Elements: array of TFloatVector) : TFloatMatrix;
var
  i,j : Integer;
begin
  SetLength(Result,Length(Elements),Length(Elements[Low(Elements)]));
  for i := Low(Elements) to High(Elements) do
    for j := Low(Elements[i]) to High(Elements[i]) do
      Result[i,j] := Elements[i,j];
end;

function CreateDoubleMatrix(Elements: array of TDoubleVector) : TDoubleMatrix;
var
  i,j : Integer;
begin
  SetLength(Result,Length(Elements),Length(Elements[Low(Elements)]));
  for i := Low(Elements) to High(Elements) do
    for j := Low(Elements[i]) to High(Elements[i]) do
      Result[i,j] := Elements[i,j];
end;

function CreateByteColor(Elements: array of Byte) : TByteColor;
var
  i : Integer;
begin
  SetLength(Result, Length(Elements));
  for i := Low(Elements) to High(Elements) do
    Result[i] := Elements[i];
end;

function CreateWordColor(Elements: array of Word) : TWordColor;
var
  i : Integer;
begin
  SetLength(Result, Length(Elements));
  for i := Low(Elements) to High(Elements) do
    Result[i] := Elements[i];
end;

function CreateFloatColor(Elements: array of Float) : TFloatColor;
var
  i : Integer;
begin
  SetLength(Result, Length(Elements));
  for i := Low(Elements) to High(Elements) do
    Result[i] := Elements[i];
end;




function ConvertVector(Vector: TVector2f) : TFloatVector; overload
var
  i : Integer;
begin
  SetLength(Result,Length(Vector));
  for i := Low(Vector) to High(Vector) do
    Result[i] := Vector[i];
end;

function ConvertVector(Vector: TVector3f) : TFloatVector; overload
var
  i : Integer;
begin
  SetLength(Result,Length(Vector));
  for i := Low(Vector) to High(Vector) do
    Result[i] := Vector[i];
end;

function ConvertVector(Vector: TVector4f) : TFloatVector; overload
var
  i : Integer;
begin
  SetLength(Result,Length(Vector));
  for i := Low(Vector) to High(Vector) do
    Result[i] := Vector[i];
end;

function ConvertVector(Vector: TVector2d) : TDoubleVector; overload
var
  i : Integer;
begin
  SetLength(Result,Length(Vector));
  for i := Low(Vector) to High(Vector) do
    Result[i] := Vector[i];
end;

function ConvertVector(Vector: TVector3d) : TDoubleVector; overload
var
  i : Integer;
begin
  SetLength(Result,Length(Vector));
  for i := Low(Vector) to High(Vector) do
    Result[i] := Vector[i];
end;

function ConvertVector(Vector: TVector4d) : TDoubleVector; overload
var
  i : Integer;
begin
  SetLength(Result,Length(Vector));
  for i := Low(Vector) to High(Vector) do
    Result[i] := Vector[i];
end;


// proof of concept:

function AddVector(const v1,v2: TFloatVector) : TFloatVector; overload;
var
  l : Integer;
begin
  l := Length(v1);
  if l = Length(v2) then
  begin
    SetLength(Result,l);
    case l of
      2 : AddVector2f(TVector2f(Addr(v1[0])^), TVector2f(Addr(v2[0])^), TVector2f(Addr(Result[0])^));
      3 : AddVector3f(TVector3f(Addr(v1[0])^), TVector3f(Addr(v2[0])^), TVector3f(Addr(Result[0])^));
      4 : AddVector4f(TVector4f(Addr(v1[0])^), TVector4f(Addr(v2[0])^), TVector4f(Addr(Result[0])^));
      else {error}
    end;
  end
  else {error}
end;

function AddVector(const v1,v2: TDoubleVector) : TDoubleVector; overload;
var
  i : Integer;
begin
  if Length(v1) = Length(v2) then
  begin
    SetLength(Result,Length(v1));
    for i := Low(v1) to High(v1) do
      Result[i] := v1[i] + v2[i];
  end
  else {error}
end;

end.
